{********************************************************************************}
{ Ajedrez 1.0 - Juego del ajedrez bajo ms-dos.                                   }
{ Copyright (C) 2005  Gorka Surez Garca                                        }
{                                                                                }
{ IAJuego.pas is part of "Ajedrez".                                              }
{                                                                                }
{ "Ajedrez" is free software; you can redistribute it and/or                     }
{ modify it under the terms of the GNU General Public License                    }
{ as published by the Free Software Foundation; either version 2                 }
{ of the License, or (at your option) any later version.                         }
{                                                                                }
{ "Ajedrez" is distributed in the hope that it will be useful,                   }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of                 }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  }
{ GNU General Public License for more details.                                   }
{                                                                                }
{ You should have received a copy of the GNU General Public License              }
{ along with "Ajedrez"; if not, write to the Free Software                       }
{ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.    }
{********************************************************************************}
unit IAJuego;
{********************************************************************************}
interface {parte publica}
uses Lesslib, Nucleo, LogMovs, crt;
{********************************************************************************}
{ Funciones para controlar la inteligencia artificial del juego.                 }
{********************************************************************************}
procedure MaquinaMueve (var Nucleo: tNucleo);

function Maximizador (var Nucleo: tNucleo; color, profundidad: byte;
                      alpha, beta: longint): longint;
function Minimizador (var Nucleo: tNucleo; color, profundidad: byte;
                      alpha, beta: longint): longint;

{********************************************************************************}
implementation {parte privada}
{********************************************************************************}


{********************************************************************************}
{ Constantes exclusivas de la IA.                                                }
{********************************************************************************}
const
   ALPHA_INICIAL = $80000000;
   BETA_INICIAL  = $7FFFFFFF;

   MAX_PROFUNDIDAD  = 2;
   

   IA_VICTORIA = $7FFFFFFF;
   IA_DERROTA  = $80000000;

   VALOR_PEON    =       1;
   VALOR_TORRE   =       5;
   VALOR_CABALLO =       3;
   VALOR_ALFIL   =       3;
   VALOR_REINA   =       9;
   VALOR_MOVREY  =     100;
   VALOR_REY     =    1000;

   VALOR_MOVIMIENTO =  1;
   VALOR_AMENAZAS   = 16;
   VALOR_VIVAS      = 64;

{********************************************************************************}
{ Tipos estructurados exclusivos de la IA.                                       }
{********************************************************************************}
type
   {-----------------------------------------------------------------------------}
   { Fichas Capaces de Realizar el Movimiento.                                   }
   {-----------------------------------------------------------------------------}
   tFichasCRM = array[1..MAX_FICHAS_JUG] of boolean;

{********************************************************************************}
{ Funcin que determina que piezas del contrario amenazan una casilla.           }
{********************************************************************************}
procedure AmenazadaPor (var Nucleo: tNucleo; color, ficha: byte; var fichas: tFichasCRM);
var
   tabaux: tFichas;
   i: integer;
   cene: byte;
begin
   {-----------------------------------------------------------------------------}
   { Guardamos el contenido actual del tablero.                                  }
   {-----------------------------------------------------------------------------}
   tabaux := Nucleo.Juego.Fichas;

   {-----------------------------------------------------------------------------}
   { Echamos un vistazo a ver quien es el color enemigo.                         }
   {-----------------------------------------------------------------------------}
   if color = FICHA_NEGRA then
      cene := FICHA_BLANCA
   else
      cene := FICHA_NEGRA;

   {-----------------------------------------------------------------------------}
   { Y vamos ficha por ficha del enemigo, comprobando primero si est viva y     }
   { segundo si esta puede moverse a la posicin de la actual.                   }
   {-----------------------------------------------------------------------------}
   for i := 1 to MAX_FICHAS_JUG do
      if Nucleo.Juego.Fichas[cene, i].Tipo <> FICHA_MUERTA then begin
         fichas[i] := ValidarMovimiento(Nucleo, cene, i,
                                        Nucleo.Juego.Fichas[color, ficha].Columna,
                                        Nucleo.Juego.Fichas[color, ficha].Fila);

         Nucleo.Juego.Fichas := tabaux;
      end else begin
         fichas[i] := false;
      end;
end;

{********************************************************************************}
{ Funcin que determina que piezas de la maquina defienden una casilla.          }
{********************************************************************************}
procedure DefendidaPor (var Nucleo: tNucleo; color, ficha: byte; var fichas: tFichasCRM);
var
   tabaux: tFichas;
   i: integer;
   fic, col: byte;
begin
   {-----------------------------------------------------------------------------}
   { Guardamos el contenido actual del tablero.                                  }
   {-----------------------------------------------------------------------------}
   tabaux := Nucleo.Juego.Fichas;

   {-----------------------------------------------------------------------------}
   { Suponemos que ningn pen nos protege inicialmente.                         }
   {-----------------------------------------------------------------------------}
   for i := 1 to MAX_PEONES do
      fichas[i] := false;

   {-----------------------------------------------------------------------------}
   { Pero para comprobarlo de forma sencilla echaremos un vistazo a las casillas }
   { que hay cerca de la indicada, pero dependiendo del color las coordenadas    }
   { sern unas o sern otras.                                                   }
   {-----------------------------------------------------------------------------}
   if color = FICHA_NEGRA then begin
      {--------------------------------------------------------------------------}
      { Miramos la casilla superior izquierda.                                   }
      {--------------------------------------------------------------------------}
      ObtenerFicha(Nucleo, Nucleo.Juego.Fichas[color, ficha].Columna - 1,
                   Nucleo.Juego.Fichas[color, ficha].Fila + 1, fic, col);

      if (col = FICHA_NEGRA) and (0 < fic) and (fic <= MAX_PEONES) then
         fichas[fic] := true;

      {--------------------------------------------------------------------------}
      { Miramos la casilla superior derecha.                                     }
      {--------------------------------------------------------------------------}
      ObtenerFicha(Nucleo, Nucleo.Juego.Fichas[color, ficha].Columna + 1,
                   Nucleo.Juego.Fichas[color, ficha].Fila + 1, fic, col);

      if (col = FICHA_NEGRA) and (0 < fic) and (fic <= MAX_PEONES) then
         fichas[fic] := true;
   end else begin
      {--------------------------------------------------------------------------}
      { Miramos la casilla inferior izquierda.                                   }
      {--------------------------------------------------------------------------}
      ObtenerFicha(Nucleo, Nucleo.Juego.Fichas[color, ficha].Columna - 1,
                   Nucleo.Juego.Fichas[color, ficha].Fila - 1, fic, col);

      if (col = FICHA_BLANCA) and (0 < fic) and (fic <= MAX_PEONES) then
         fichas[fic] := true;

      {--------------------------------------------------------------------------}
      { Miramos la casilla inferior derecha.                                     }
      {--------------------------------------------------------------------------}
      ObtenerFicha(Nucleo, Nucleo.Juego.Fichas[color, ficha].Columna + 1,
                   Nucleo.Juego.Fichas[color, ficha].Fila - 1, fic, col);

      if (col = FICHA_BLANCA) and (0 < fic) and (fic <= MAX_PEONES) then
         fichas[fic] := true;
   end;
   
   {-----------------------------------------------------------------------------}
   { Comprobamos el resto de las fichas del bando.                               }
   {-----------------------------------------------------------------------------}
   for i := TORRE_IZQ to MAX_FICHAS_JUG do begin
      {--------------------------------------------------------------------------}
      { La ficha que est en la casilla que deseamos comprobar si est defendida }
      { la ponemos como muerta, para evitar problemas con la funcin que valida  }
      { el movimiento de las piezas.                                             }
      {--------------------------------------------------------------------------}
      Nucleo.Juego.Fichas[color, ficha].Tipo := FICHA_MUERTA;

      {--------------------------------------------------------------------------}
      { Si la ficha actual no est muerta, miramos a ver si defiende el lugar.   }
      {--------------------------------------------------------------------------}
      if Nucleo.Juego.Fichas[color, i].Tipo <> FICHA_MUERTA then begin
         fichas[i] := ValidarMovimiento(Nucleo, color, i,
                                        Nucleo.Juego.Fichas[color, ficha].Columna,
                                        Nucleo.Juego.Fichas[color, ficha].Fila);
                                        
      {--------------------------------------------------------------------------}
      { Sino est viva obviamente no defiende la casilla.                        }
      {--------------------------------------------------------------------------}
      end else begin
         fichas[i] := false;
      end;

      {--------------------------------------------------------------------------}
      { Recuperamos el tablero como estaba antes.                                }
      {--------------------------------------------------------------------------}
      Nucleo.Juego.Fichas := tabaux;
   end;
end;

{********************************************************************************}
{ Funcin que nos devuelve el nmero de fichas activas del tipo tFichasCRM.      }
{********************************************************************************}
function NumFichasCRM (fichas: tFichasCRM) : byte;
var
   i, result: byte;
begin
   result := 0;

   for i := 1 to MAX_FICHAS_JUG do
      if fichas[i] then result := result + 1;

   NumFichasCRM := result;
end;

{********************************************************************************}
{ Esta funcin devuelve el valor de un tipo de ficha segn su tipo.              }
{********************************************************************************}
function DevolverValorFicha (var Nucleo: tNucleo; color, ficha: byte) : integer;
begin
   if Nucleo.Juego.Fichas[color, ficha].Tipo <> FICHA_MUERTA then begin
      case ficha of
         1..MAX_PEONES: begin
            case Nucleo.Juego.Fichas[color, ficha].Tipo of
               FICHA_PEON:    DevolverValorFicha := VALOR_PEON;
               FICHA_TORRE:   DevolverValorFicha := VALOR_TORRE;
               FICHA_CABALLO: DevolverValorFicha := VALOR_CABALLO;
               FICHA_ALFIL:   DevolverValorFicha := VALOR_ALFIL;
               FICHA_REINA:   DevolverValorFicha := VALOR_REINA;
            end;
         end;

         TORRE_IZQ, TORRE_DER:     DevolverValorFicha := VALOR_TORRE;
         CABALLO_IZQ, CABALLO_DER: DevolverValorFicha := VALOR_CABALLO;
         ALFIL_IZQ, ALFIL_DER:     DevolverValorFicha := VALOR_ALFIL;
         REINA:                    DevolverValorFicha := VALOR_REINA;
         REY:                      DevolverValorFicha := VALOR_MOVREY;
      end;
   end else begin
      DevolverValorFicha := 0;
   end;
end;

{********************************************************************************}
{ Esta funcin sirve para recorrer de menor a mayor valor las fichas.            }
{********************************************************************************}
function DarPosRealMenorMayor (i: byte): byte;
begin
   case i of
      1..8: DarPosRealMenorMayor := i;
      9:    DarPosRealMenorMayor := CABALLO_IZQ;
      10:   DarPosRealMenorMayor := ALFIL_IZQ;
      11:   DarPosRealMenorMayor := CABALLO_DER;
      12:   DarPosRealMenorMayor := ALFIL_DER;
      13:   DarPosRealMenorMayor := TORRE_IZQ;
      14:   DarPosRealMenorMayor := TORRE_DER;
      15:   DarPosRealMenorMayor := REINA;
      16:   DarPosRealMenorMayor := REY;
   end;
end;

{********************************************************************************}
{ Esta funcin servir para determinar si una ficha est en claro peligro.       }
{********************************************************************************}
function CorrePeligro (var Nucleo: tNucleo; color, ficha: byte) : boolean;
var
   amenaza, defensa: tFichasCRM;
   numame, numdef, i, j, cene: byte;
   perdida, ganancia, valmia, valene: integer;
   salir: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Obtenemos las fichas que nos amenazan, y tambin cuantas son.               }
   {-----------------------------------------------------------------------------}
   AmenazadaPor(Nucleo, color, ficha, amenaza);
   numame := NumFichasCRM(amenaza);

   {-----------------------------------------------------------------------------}
   { Si nos amenazan cero, no hay peligro.                                       }
   {-----------------------------------------------------------------------------}
   if numame = 0 then begin
      CorrePeligro := false;
   {-----------------------------------------------------------------------------}
   { Sino, tenemos que ver la situacin en ms detalle.                          }
   {-----------------------------------------------------------------------------}
   end else begin
      {--------------------------------------------------------------------------}
      { Obtenemos las fichas que nos defienden, y tambin cuantas son.           }
      {--------------------------------------------------------------------------}
      DefendidaPor(Nucleo, color, ficha, defensa);
      numdef := NumFichasCRM(defensa);

      {--------------------------------------------------------------------------}
      { Si nos defienden cero, sin duda alguna hay peligro.                      }
      {--------------------------------------------------------------------------}
      if numdef = 0 then begin
         CorrePeligro := true;
      {--------------------------------------------------------------------------}
      { Sino entraremos en ms detalle todava.                                  }
      {--------------------------------------------------------------------------}
      end else begin
         {-----------------------------------------------------------------------}
         { Dado que hay al menos una ficha que nos amenaza, aadiremos a las     }
         { perdidas la ficha principal que tenemos en observacin.               }
         {-----------------------------------------------------------------------}
         valmia := DevolverValorFicha(Nucleo, color, ficha);
         i := 1; j := 1; salir := false;

         {-----------------------------------------------------------------------}
         { Echamos un vistazo a ver quien es el bando contrario.                 }
         {-----------------------------------------------------------------------}
         if color = FICHA_BLANCA then
            cene := FICHA_NEGRA
         else if color = FICHA_NEGRA then
            cene := FICHA_BLANCA;

         {-----------------------------------------------------------------------}
         { Y ya que nos amenaza al menos una, la buscamos.                       }
         {-----------------------------------------------------------------------}
         while (amenaza[DarPosRealMenorMayor(i)] = false) do i := i + 1;
         valene := DevolverValorFicha(Nucleo, cene, DarPosRealMenorMayor(i));

         {-----------------------------------------------------------------------}
         { Y nos metemos en un bucle que terminar cuando se agote alguna de las }
         { fichas, o las perdidas y las ganancias sean distintas.                }
         {-----------------------------------------------------------------------}
         repeat
            {--------------------------------------------------------------------}
            { Si la enemiga es de menor o igual valor.                           }
            {--------------------------------------------------------------------}
            if (valene <= valmia) then begin
               {-----------------------------------------------------------------}
               { Aumentamos las perdidas de la mquina.                          }
               {-----------------------------------------------------------------}
               perdida := perdida + valmia; i := i + 1;

               {-----------------------------------------------------------------}
               { Buscamos la siguiente ficha de la defensa disponible.           }
               {-----------------------------------------------------------------}
               while (j <= MAX_FICHAS_JUG) and
                     (defensa[DarPosRealMenorMayor(j)] = false) do
                  j := j + 1;

               {-----------------------------------------------------------------}
               { Si tenemos alguna ficha echamos un vistazo a la situacin,      }
               { almacenando primero el valor de esta ficha.                     }
               {-----------------------------------------------------------------}
               if (j <= MAX_FICHAS_JUG) then begin
                  valmia := DevolverValorFicha(Nucleo, color, DarPosRealMenorMayor(j));
               
                  {--------------------------------------------------------------}
                  { Buscamos la siguiente ficha que nos amenaza.                 }
                  {--------------------------------------------------------------}
                  while (i <= MAX_FICHAS_JUG) and
                        (amenaza[DarPosRealMenorMayor(i)] = false) do
                     i := i + 1;
            
                  {--------------------------------------------------------------}
                  { Si hay alguna ms que nos amenace, nos lo pensamos, es decir }
                  { si es una ficha de menor valor la comenos, sino salimos.     }
                  {--------------------------------------------------------------}
                  if (i <= MAX_FICHAS_JUG) then begin
                     if (valene >= valmia) then begin
                        ganancia := ganancia + valene; j := j + 1;
                        valene := DevolverValorFicha(Nucleo, cene, DarPosRealMenorMayor(i));
                     end else begin
                        salir := true;
                     end;
                  {--------------------------------------------------------------}
                  { Si no quedan fichas que amenacen, la comemos y salimos.      }
                  {--------------------------------------------------------------}
                  end else begin
                     ganancia := ganancia + valene;
                     salir := true;
                  end;
               {-----------------------------------------------------------------}
               { Sino tenemos defensas salimos del bucle.                        }
               {-----------------------------------------------------------------}
               end else begin
                  salir := true;
               end;
            {--------------------------------------------------------------------}
            { Si es de mayor valor miramos que no le queden defensas a la        }
            { mquina, y de ser as se la comemos.                               }
            {--------------------------------------------------------------------}
            end else begin
               if (j > MAX_FICHAS_JUG) then begin
                  perdida := perdida + valmia;
               end;

               salir := true;
            end;
         until salir;

         {-----------------------------------------------------------------------}
         { Si las perdidas son mayores que las ganancias, estamos en peligro.    }
         {-----------------------------------------------------------------------}
         if perdida > ganancia then
            CorrePeligro := true
         {-----------------------------------------------------------------------}
         { Si son iguales o menores, no hay peligro real.                        }
         {-----------------------------------------------------------------------}
         else
            CorrePeligro := false;
      end;
   end;
end;

{********************************************************************************}
{ Funcin que puntua el grado de amenaza que tiene un bando.                     }
{********************************************************************************}
function EvaluarAmenazas (var Nucleo: tNucleo; color: byte): longint;
var
   i: byte;
   result: longint;
begin
   result := 0;

   for i := 1 to MAX_FICHAS_JUG do begin
      if (Nucleo.Juego.Fichas[color, i].Tipo <> FICHA_MUERTA) and
         CorrePeligro(Nucleo, color, i) then begin
         case Nucleo.Juego.Fichas[color, i].Tipo of
            FICHA_PEON:    result := result + VALOR_PEON;
            FICHA_TORRE:   result := result + VALOR_TORRE;
            FICHA_CABALLO: result := result + VALOR_CABALLO;
            FICHA_ALFIL:   result := result + VALOR_ALFIL;
            FICHA_REINA:   result := result + VALOR_REINA;
            {FICHA_REY:    result := result + VALOR_REY;}
         end;
      end;
   end;

   EvaluarAmenazas := result;
end;

{********************************************************************************}
{ Funcin que puntua las muertes producidas en el juego.                         }
{********************************************************************************}
function EvaluarVivas (var Nucleo: tNucleo; color: byte): longint;
var
   i: byte;
   result: longint;
begin
   result := 0;

   for i := 1 to MAX_FICHAS_JUG do begin
      case Nucleo.Juego.Fichas[color, i].Tipo of
         FICHA_PEON:    result := result + VALOR_PEON;
         FICHA_TORRE:   result := result + VALOR_TORRE;
         FICHA_CABALLO: result := result + VALOR_CABALLO;
         FICHA_ALFIL:   result := result + VALOR_ALFIL;
         FICHA_REINA:   result := result + VALOR_REINA;
         FICHA_REY:     result := result + VALOR_REY;
      end;
   end;

   EvaluarVivas := result;
end;

{********************************************************************************}
{ Funcin que devuelve el valor de las casillas del tablero.                     }
{********************************************************************************}
function DarValorCasTabNormal (var Nucleo: tNucleo; color, ficha: byte): integer;
var
   result: integer;
begin
   result := 0;

   if Nucleo.Juego.Fichas[color, ficha].Tipo <> FICHA_REY then begin
      if color = TURNO_BLANCAS then begin
         DarValorCasTabNormal := Nucleo.Juego.Fichas[color, ficha].Fila - 1;
      end else begin
         DarValorCasTabNormal := 8 - Nucleo.Juego.Fichas[color, ficha].Fila;
      end;
   end;

   DarValorCasTabNormal := result;
end;
{--------------------------------------------------------------------------------}
function DarDistMayorReyNvsReyB (var Nucleo: tNucleo): integer;
var
   diff, difc: byte;
begin
   diff := abs(Nucleo.Juego.Fichas[FICHA_BLANCA, REY].Fila -
               Nucleo.Juego.Fichas[FICHA_NEGRA, REY].Fila);
   difc := abs(Nucleo.Juego.Fichas[FICHA_BLANCA, REY].Columna -
               Nucleo.Juego.Fichas[FICHA_NEGRA, REY].Columna);

   if diff > difc then
      DarDistMayorReyNvsReyB := diff
   else
      DarDistMayorReyNvsReyB := difc;
end;
{--------------------------------------------------------------------------------}
function DarDistEntreReinavsReyB (var Nucleo: tNucleo; ficha: byte): integer;
var
   diff, difc: byte;
begin
   diff := abs(Nucleo.Juego.Fichas[FICHA_BLANCA, REY].Fila -
               Nucleo.Juego.Fichas[FICHA_NEGRA, ficha].Fila);
   difc := abs(Nucleo.Juego.Fichas[FICHA_BLANCA, REY].Columna -
               Nucleo.Juego.Fichas[FICHA_NEGRA, ficha].Columna);

   if (diff <= 1) and (difc <= 1) then
      DarDistEntreReinavsReyB:= -VALOR_REINA * 10
   else
      DarDistEntreReinavsReyB:= VALOR_REINA * 10;
end;
{--------------------------------------------------------------------------------}
function DarValorCasTabReinaTorre (var Nucleo: tNucleo; color, ficha: byte): integer;
begin
   if (color = TURNO_BLANCAS) and (ficha = REY) then begin
      if (Nucleo.Juego.Fichas[color, ficha].Fila = 1) or
         (Nucleo.Juego.Fichas[color, ficha].Fila = 8) or
         (Nucleo.Juego.Fichas[color, ficha].Columna = 1) or
         (Nucleo.Juego.Fichas[color, ficha].Columna = 8) then begin
         DarValorCasTabReinaTorre := 0;

      end else if (Nucleo.Juego.Fichas[color, ficha].Fila = 2) or
                  (Nucleo.Juego.Fichas[color, ficha].Fila = 7) or
                  (Nucleo.Juego.Fichas[color, ficha].Columna = 2) or
                  (Nucleo.Juego.Fichas[color, ficha].Columna = 7) then begin
         DarValorCasTabReinaTorre := 10;

      end else if (Nucleo.Juego.Fichas[color, ficha].Fila = 3) or
                  (Nucleo.Juego.Fichas[color, ficha].Fila = 6) or
                  (Nucleo.Juego.Fichas[color, ficha].Columna = 3) or
                  (Nucleo.Juego.Fichas[color, ficha].Columna = 6) then begin
         DarValorCasTabReinaTorre := 20;

      end else if (Nucleo.Juego.Fichas[color, ficha].Fila = 4) or
                  (Nucleo.Juego.Fichas[color, ficha].Fila = 5) or
                  (Nucleo.Juego.Fichas[color, ficha].Columna = 5) or
                  (Nucleo.Juego.Fichas[color, ficha].Columna = 5) then begin
         DarValorCasTabReinaTorre := 40;

      end else begin
         DarValorCasTabReinaTorre := 80;

      end;
   end else if (color = TURNO_NEGRAS) and
               (Nucleo.Juego.Fichas[color, ficha].Tipo = FICHA_REINA) then begin
      DarValorCasTabReinaTorre := DarDistEntreReinavsReyB(Nucleo, ficha);
{
   end else if (color = TURNO_NEGRAS) and
               (Nucleo.Juego.Fichas[color, ficha].Tipo = FICHA_TORRE) then begin
      DarValorCasTabReinaTorre := DarDistEntreReinavsReyB(Nucleo, ficha);
}
   end else if (color = TURNO_NEGRAS) and (ficha = REY) then begin
      DarValorCasTabReinaTorre := -DarDistMayorReyNvsReyB(Nucleo);
   end;
end;
{--------------------------------------------------------------------------------}
function DarValorCasillaTablero (var Nucleo: tNucleo; color, ficha: byte): integer;
begin
   case Nucleo.Juego.EstadoIA of
      ESTADOIA_NORMAL:   DarValorCasillaTablero := DarValorCasTabNormal(Nucleo, color, ficha);
      ESTADOIA_FINREINA: DarValorCasillaTablero := DarValorCasTabReinaTorre(Nucleo, color, ficha);
      ESTADOIA_FINTORRE: DarValorCasillaTablero := DarValorCasTabReinaTorre(Nucleo, color, ficha);
      ESTADOIA_FINALFIL: DarValorCasillaTablero := DarValorCasTabNormal(Nucleo, color, ficha);
   end;
end;

{********************************************************************************}
{ Funcin que puntua las posiciones de las fichas en el tablero.                 }
{********************************************************************************}
function EvaluarPosiciones (var Nucleo: tNucleo; color: byte): longint;
var
   i: byte;
   result: longint;
begin
   result := 0;

   for i := 1 to MAX_FICHAS_JUG do begin
      case Nucleo.Juego.Fichas[color, i].Tipo of
         FICHA_PEON:    result := result + VALOR_PEON * DarValorCasillaTablero(Nucleo, color, i);
         FICHA_TORRE:   result := result + VALOR_TORRE * DarValorCasillaTablero(Nucleo, color, i);
         FICHA_CABALLO: result := result + VALOR_CABALLO * DarValorCasillaTablero(Nucleo, color, i);
         FICHA_ALFIL:   result := result + VALOR_ALFIL * DarValorCasillaTablero(Nucleo, color, i);
         FICHA_REINA:   result := result + VALOR_REINA * DarValorCasillaTablero(Nucleo, color, i);
         FICHA_REY:     result := result + VALOR_PEON * DarValorCasillaTablero(Nucleo, color, i);
      end;
   end;

   EvaluarPosiciones := result;
end;

{********************************************************************************}
{ Funcin que nos devuelve una evaluacin simple del tablero actual.             }
{********************************************************************************}
function EvaluarTablero (var Nucleo: tNucleo): longint;
var
   puntbla, puntneg: integer;
begin
   {-----------------------------------------------------------------------------}
   { En caso de que no haya victoria evaluamos la situacin de forma normal.     }
   {-----------------------------------------------------------------------------}
   if Nucleo.Juego.Victoria = VICTORIA_NO then begin
      {--------------------------------------------------------------------------}
      { Echamos un vistazo para ver las muertes producidas por cada bando.       }
      {--------------------------------------------------------------------------}
      puntbla := EvaluarVivas(Nucleo, FICHA_BLANCA) * VALOR_VIVAS;
      puntneg := EvaluarVivas(Nucleo, FICHA_NEGRA)  * VALOR_VIVAS;

      {--------------------------------------------------------------------------}
      { Echamos un vistazo para ver las posiciones tomadas por cada bando.       }
      {--------------------------------------------------------------------------}
      puntbla := puntbla + EvaluarPosiciones(Nucleo, FICHA_BLANCA) * VALOR_MOVIMIENTO;
      puntneg := puntneg + EvaluarPosiciones(Nucleo, FICHA_NEGRA)  * VALOR_MOVIMIENTO;

      {--------------------------------------------------------------------------}
      { Echamos un vistazo para ver las fichas amenazadas por cada bando.        }
      {--------------------------------------------------------------------------}
      puntbla := puntbla + EvaluarAmenazas(Nucleo, FICHA_NEGRA)  * VALOR_AMENAZAS;
      puntneg := puntneg + EvaluarAmenazas(Nucleo, FICHA_BLANCA) * VALOR_AMENAZAS;

      {--------------------------------------------------------------------------}
      { Y la evaluacin ser el nmero de negras, menos las blancas.             }
      {--------------------------------------------------------------------------}
      EvaluarTablero := puntneg - puntbla;

   {-----------------------------------------------------------------------------}
   { En caso de que gane el jugador, devolvemos el valor para la derrota.        }
   {-----------------------------------------------------------------------------}
   end else if Nucleo.Juego.Victoria = VICTORIA_J1 then begin
      EvaluarTablero := IA_DERROTA;

   {-----------------------------------------------------------------------------}
   { En caso de que gane la maquina, devolvemos el valor para la victoria.       }
   {-----------------------------------------------------------------------------}
   end else if Nucleo.Juego.Victoria = VICTORIA_J2 then begin
      EvaluarTablero := IA_VICTORIA;

   {-----------------------------------------------------------------------------}
   { El otro caso sera las tablas y a efectos prcticos es como una derrota.    }
   {-----------------------------------------------------------------------------}
   end else begin
      EvaluarTablero := IA_DERROTA;
   end;
end;

{********************************************************************************}
{ Funcin que nos devuelve el bando que le tocar mover en el siguiente turno.   }
{********************************************************************************}
function DarSigTurno (color: byte): byte;
begin
   {-----------------------------------------------------------------------------}
   { Si es el turno de las blancas, el prximo ser el de las negras.            }
   {-----------------------------------------------------------------------------}
   if color = TURNO_BLANCAS then
      DarSigTurno := TURNO_NEGRAS
   {-----------------------------------------------------------------------------}
   { Sino es el turno de las negras, y el prximo ser el de las blancas.        }
   {-----------------------------------------------------------------------------}
   else
      DarSigTurno := TURNO_BLANCAS;
end;

{********************************************************************************}
{ Funcin que sirve para escoger la mejor jugada para la mquina.                }
{********************************************************************************}
function Maximizador (var Nucleo: tNucleo; color, profundidad: byte;
                      alpha, beta: longint): longint;
var
   ficha, c, f, i: byte;
   aux: tJuego;
   valormax, valoract: longint;
   salir: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Determinamos si hay bloqueo, de haberlo "terminara" la partida.            }
   {-----------------------------------------------------------------------------}
   CambiarTurno(Nucleo);
   Nucleo.Juego.NoPeon := true;

   {-----------------------------------------------------------------------------}
   { Indicamos que el valor mximo todava no est inicializado.                 }
   {-----------------------------------------------------------------------------}
   valormax := ALPHA_INICIAL;

   {-----------------------------------------------------------------------------}
   { Si hemos sobrepasado la profundidad del "arbol" de bsqueda, o si la        }
   { partida ha terminado, devolvemos la evaluacin del tablero actual.          }
   {-----------------------------------------------------------------------------}
   if (MAX_PROFUNDIDAD < profundidad) or
      (Nucleo.Juego.Victoria <> VICTORIA_NO) or
      (Nucleo.Juego.JaqueB) or (Nucleo.Juego.JaqueN) then begin
      
      if Nucleo.Juego.JaqueN then begin
         Maximizador := IA_DERROTA + profundidad;
      end else if Nucleo.Juego.JaqueB then begin
         Maximizador := IA_VICTORIA - profundidad;
      end else begin
         Maximizador := EvaluarTablero(Nucleo);
      end;
      
   {-----------------------------------------------------------------------------}
   { Sino echamos un vistazo a todas las posibles jugadas.                       }
   {-----------------------------------------------------------------------------}
   end else begin
      {--------------------------------------------------------------------------}
      { Salvamos el tablero y ponemos a falso el flag de salir del bucle.        }
      {--------------------------------------------------------------------------}
      aux   := Nucleo.Juego;
      salir := false;

      {--------------------------------------------------------------------------}
      { Recorremos todo el tablero probando ficha por ficha, a fin de            }
      { encontrar todas las posibles jugadas del turno actual.                   }
      {--------------------------------------------------------------------------}
      for ficha := 1 to MAX_FICHAS_JUG do begin
         if Nucleo.Juego.Fichas[color, ficha].Tipo <> FICHA_MUERTA then begin
            for i := 1 to Nucleo.MovsFichas.NumMovs[Nucleo.Juego.Fichas[color, ficha].Tipo] do begin
               DarMovimiento(Nucleo, i, color, ficha, c, f);
               if (f > 0) and (c > 0) and ValidarMovimiento(Nucleo, color, ficha, c, f) then begin
                  {--------------------------------------------------------------}
                  { Pedimos el valor mnimo de las siguientes jugadas.           }
                  {--------------------------------------------------------------}
                  valoract := Minimizador(Nucleo, DarSigTurno(color),
                                          profundidad + 1, alpha, beta);

                  {--------------------------------------------------------------}
                  { Restauramos el tablero.                                      }
                  {--------------------------------------------------------------}
                  Nucleo.Juego := aux;

                  {--------------------------------------------------------------}
                  { Y echamos un vistazo a la situacin en s. Si est sin       }
                  { inicializar, o si el valor mximo actual es menor que el que }
                  { acabamos de conseguir, entonces actualizamos los valores. Y  }
                  { adems cambiamos el valor de alpha, que es la cota inferior  }
                  { que puede asignarse en ltimo termino al nodo maximizante.   }
                  {--------------------------------------------------------------}
                  if (valormax < valoract) then begin
                     valormax := valoract;

                     alpha := valoract;
                  end;

                  {--------------------------------------------------------------}
                  { Si la cota superior del mnimo, es mayor que la inferior     }
                  { del mximo, dejamos de comprobar los siguientes terminos.    }
                  {--------------------------------------------------------------}
                  if (alpha > beta) then begin
                     salir := true;
                     break;
                  end;
               end;
            end;

            if salir then break;
         end;
      end;

      {--------------------------------------------------------------------------}
      { Devolvemos el valor mximo obtenido.                                     }
      {--------------------------------------------------------------------------}
      Maximizador := valormax;
   end;
end;

{********************************************************************************}
{ Funcin que sirve para escoger la peor jugada para la mquina.                 }
{********************************************************************************}
function Minimizador (var Nucleo: tNucleo; color, profundidad: byte;
                      alpha, beta: longint): longint;
var
   ficha, c, f, i: byte;
   aux: tJuego;
   valormin, valoract: longint;
   salir: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Determinamos si hay bloqueo, de haberlo "terminara" la partida.            }
   {-----------------------------------------------------------------------------}
   CambiarTurno(Nucleo);
   Nucleo.Juego.NoPeon := true;

   {-----------------------------------------------------------------------------}
   { Indicamos que el valor mnimo todava no est inicializado.                 }
   {-----------------------------------------------------------------------------}
   valormin := BETA_INICIAL;

   {-----------------------------------------------------------------------------}
   { Si hemos sobrepasado la profundidad del "arbol" de bsqueda, o si la        }
   { partida ha terminado, devolvemos la evaluacin del tablero actual.          }
   {-----------------------------------------------------------------------------}
   if (MAX_PROFUNDIDAD < profundidad) or
      (Nucleo.Juego.Victoria <> VICTORIA_NO) or
      (Nucleo.Juego.JaqueB) or (Nucleo.Juego.JaqueN) then begin

      if Nucleo.Juego.JaqueN then begin
         Minimizador := IA_DERROTA + profundidad;
      end else if Nucleo.Juego.JaqueB then begin
         Minimizador := IA_VICTORIA - profundidad;
      end else begin
         Minimizador := EvaluarTablero(Nucleo);
      end;
      
   {-----------------------------------------------------------------------------}
   { Sino echamos un vistazo a todas las posibles jugadas.                       }
   {-----------------------------------------------------------------------------}
   end else begin
      {--------------------------------------------------------------------------}
      { Salvamos el tablero y ponemos a falso el flag de salir del bucle.        }
      {--------------------------------------------------------------------------}
      aux   := Nucleo.Juego;
      salir := false;
      
      {--------------------------------------------------------------------------}
      { Recorremos todo el tablero probando ficha por ficha, a fin de            }
      { encontrar todas las posibles jugadas del turno actual.                   }
      {--------------------------------------------------------------------------}
      for ficha := 1 to MAX_FICHAS_JUG do begin
         if Nucleo.Juego.Fichas[color, ficha].Tipo <> FICHA_MUERTA then begin
            for i := 1 to Nucleo.MovsFichas.NumMovs[Nucleo.Juego.Fichas[color, ficha].Tipo] do begin
               DarMovimiento(Nucleo, i, color, ficha, c, f);
               if (f > 0) and (c > 0) and ValidarMovimiento(Nucleo, color, ficha, c, f) then begin
                  {--------------------------------------------------------------}
                  { Pedimos el valor mnimo de las siguientes jugadas.           }
                  {--------------------------------------------------------------}
                  valoract := Maximizador(Nucleo, DarSigTurno(color),
                                          profundidad + 1, alpha, beta);

                  {--------------------------------------------------------------}
                  { Restauramos el tablero.                                      }
                  {--------------------------------------------------------------}
                  Nucleo.Juego := aux;

                  {--------------------------------------------------------------}
                  { Y echamos un vistazo a la situacin en s. Si est sin       }
                  { inicializar, o si el valor mnimo actual es mayor que el que }
                  { acabamos de conseguir, entonces actualizamos los valores. Y  }
                  { adems cambiamos el valor de beta, que es la cota superior   }
                  { que puede asignarse en ltimo termino al nodo minimizante.   }
                  {--------------------------------------------------------------}
                  if (valormin > valoract) then begin
                     valormin := valoract;

                     beta := valoract;
                  end;

                  {--------------------------------------------------------------}
                  { Si la cota superior del mnimo, es mayor que la inferior     }
                  { del mximo, dejamos de comprobar los siguientes terminos.    }
                  {--------------------------------------------------------------}
                  if (alpha > beta) then begin
                     salir := true;
                     break;
                  end;
               end;
            end;

            if salir then break;
         end;
      end;

      {--------------------------------------------------------------------------}
      { Devolvemos el valor mnimo obtenido.                                     }
      {--------------------------------------------------------------------------}
      Minimizador := valormin;
   end;
end;

{********************************************************************************}
{ Funcin que sirve para realizar el algoritmo del minimax para la IA.           }
{********************************************************************************}
function Minimax (var Nucleo: tNucleo; color, profundidad: byte;
                  alpha, beta: longint; var elegida, x, y: byte): longint;
var
   ficha, c, f, i: byte;
   aux: tJuego;
   valormax, valoract: longint;
begin
   {-----------------------------------------------------------------------------}
   { Inicializamos las coordenadas a devolver con 0, 0.                          }
   {-----------------------------------------------------------------------------}
   x := 0;
   y := 0;

   elegida := 0;

   {-----------------------------------------------------------------------------}
   { Si hemos sobrepasado la profundidad del "arbol" de bsqueda, o si la        }
   { partida ha terminado, devolvemos la evaluacin del tablero actual.          }
   {-----------------------------------------------------------------------------}
   if (MAX_PROFUNDIDAD < profundidad) or
      (Nucleo.Juego.Victoria <> VICTORIA_NO) then begin
      Minimax := EvaluarTablero(Nucleo);
      
   {-----------------------------------------------------------------------------}
   { Sino echamos un vistazo a todas las posibles jugadas.                       }
   {-----------------------------------------------------------------------------}
   end else begin
      {--------------------------------------------------------------------------}
      { Salvamos el tablero y ponemos a falso el flag de salir del bucle.        }
      {--------------------------------------------------------------------------}
      aux   := Nucleo.Juego;

      {--------------------------------------------------------------------------}
      { Recorremos todo el tablero probando ficha por ficha, a fin de            }
      { encontrar todas las posibles jugadas del turno actual.                   }
      {--------------------------------------------------------------------------}
      for ficha := 1 to MAX_FICHAS_JUG do begin
         if Nucleo.Juego.Fichas[color, ficha].Tipo <> FICHA_MUERTA then begin
            for i := 1 to Nucleo.MovsFichas.NumMovs[Nucleo.Juego.Fichas[color, ficha].Tipo] do begin
               DarMovimiento(Nucleo, i, color, ficha, c, f);
               if (f > 0) and (c > 0) and ValidarMovimiento(Nucleo, color, ficha, c, f) then begin
                  {--------------------------------------------------------------}
                  { Pedimos el valor mnimo de las siguientes jugadas.           }
                  {--------------------------------------------------------------}
                  valoract := Minimizador(Nucleo, DarSigTurno(color),
                                          profundidad + 1, alpha, beta);

                  {--------------------------------------------------------------}
                  { Restauramos el tablero.                                      }
                  {--------------------------------------------------------------}
                  Nucleo.Juego := aux;

                  {--------------------------------------------------------------}
                  { Y echamos un vistazo a la situacin en s. Si est sin       }
                  { inicializar, o si el valor mximo actual es menor que el que }
                  { acabamos de conseguir, entonces actualizamos los valores. Y  }
                  { adems cambiamos el valor de alpha, que es la cota inferior  }
                  { que puede asignarse en ltimo termino al nodo maximizante.   }
                  {--------------------------------------------------------------}
                  if ((x = 0) and (y = 0)) or (valormax < valoract) then begin
                     x := c;
                     y := f;

                     elegida := ficha;

                     valormax := valoract;

                     alpha := valoract;
                  end;
               end;
            end;
         end;
      end;

      {--------------------------------------------------------------------------}
      { Devolvemos el valor mximo obtenido.                                     }
      {--------------------------------------------------------------------------}
      Minimax := valormax;
   end;
end;

{********************************************************************************}
{ Calcula el estado de la partida.                                               }
{********************************************************************************}
procedure VerEstadoPartida (var Nucleo: tNucleo);
var
   i: integer;
   blancas, negras: tFichasCRM;
   sinpeones, peonreina: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Miramos que fichas estn vivas y cuales muertas.                            }
   {-----------------------------------------------------------------------------}
   for i := 9 to MAX_FICHAS_JUG do begin
      blancas[i] := (Nucleo.Juego.Fichas[FICHA_BLANCA, i].Tipo <> FICHA_MUERTA);
      negras[i]  := (Nucleo.Juego.Fichas[FICHA_NEGRA,  i].Tipo <> FICHA_MUERTA);
   end;

   {-----------------------------------------------------------------------------}
   { Para calcular si las blancas tienen peones.                                 }
   {-----------------------------------------------------------------------------}
   sinpeones := true;

   for i := 1 to MAX_PEONES do
      if Nucleo.Juego.Fichas[FICHA_BLANCA, i].Tipo <> FICHA_MUERTA then begin
         sinpeones := false;
         break;
      end;

   {-----------------------------------------------------------------------------}
   { Lo cual nos servir para saber si las blancas les queda solo el rey.        }
   {-----------------------------------------------------------------------------}
   if sinpeones and (not blancas[TORRE_IZQ]) and (not blancas[TORRE_DER]) and
      (not blancas[CABALLO_IZQ]) and (not blancas[CABALLO_DER]) and
      (not blancas[ALFIL_IZQ]) and (not blancas[ALFIL_DER]) and
      (not blancas[REINA]) then begin

      {--------------------------------------------------------------------------}
      { Miramos a ver si algn pen se convirti en reina.                       }
      {--------------------------------------------------------------------------}
      peonreina := false;
      
      for i := 1 to MAX_PEONES do
         if Nucleo.Juego.Fichas[FICHA_NEGRA, i].Tipo = FICHA_REINA then begin
            peonreina := true;
            break;
         end;

      {--------------------------------------------------------------------------}
      { Si tenemos alguna reina pasamos al modo ataque final con reina.          }
      {--------------------------------------------------------------------------}
      if negras[REINA] or peonreina then begin
         Nucleo.Juego.EstadoIA := ESTADOIA_FINREINA;
         
      {--------------------------------------------------------------------------}
      { Sino tenemos ninguna reina, pero tenemos alguna torre, pasamos al modo   }
      { ataque final con torre.                                                  }
      {--------------------------------------------------------------------------}
      end else if negras[TORRE_IZQ] or negras[TORRE_DER] then begin
         Nucleo.Juego.EstadoIA := ESTADOIA_FINTORRE;
         
      {--------------------------------------------------------------------------}
      { Si no tenemos ni reinas, ni torres, pero tenemos dos alfiles, pasamos al }
      { modo ataque final con dos alfiles.                                       }
      {--------------------------------------------------------------------------}
      end else if negras[ALFIL_IZQ] and negras[ALFIL_DER] then begin
         Nucleo.Juego.EstadoIA := ESTADOIA_FINALFIL;
         
      {--------------------------------------------------------------------------}
      { Y sino estamos jodidos.                                                  }
      {--------------------------------------------------------------------------}
      end else begin
         Nucleo.Juego.EstadoIA := ESTADOIA_NORMAL;
      end;

   {-----------------------------------------------------------------------------}
   { Si les queda algo ms que el rey, ejecutamos el algoritmo normal.           }
   {-----------------------------------------------------------------------------}
   end else begin
      Nucleo.Juego.EstadoIA := ESTADOIA_NORMAL;
   end;
end;

{********************************************************************************}
{ Funcin que gestiona los movimientos de la mquina.                            }
{********************************************************************************}
procedure MaquinaMueve (var Nucleo: tNucleo);
var
   c, f, ficha: byte;
begin
   Nucleo.Juego.NoPeon := true;

   if (Nucleo.Juego.EstadoIA = ESTADOIA_NORMAL) then
      VerEstadoPartida(Nucleo);

   {-----------------------------------------------------------------------------}
   { Buscamos la mejor coordenada posible para tirar con un mini-max.            }
   {-----------------------------------------------------------------------------}
   Minimax(Nucleo, TURNO_NEGRAS, 0, ALPHA_INICIAL, BETA_INICIAL, ficha, c, f);

   {-----------------------------------------------------------------------------}
   { Y lanzamos el movimiento que hemos determinado el mejor.                    }
   {-----------------------------------------------------------------------------}
   ValidarMovimiento(Nucleo, TURNO_NEGRAS, ficha, c, f);

   Nucleo.Juego.NoPeon := false;
end;

{********************************************************************************}
end.
{********************************************************************************}