{********************************************************************************}
{ Ajedrez 1.0 - Juego del ajedrez bajo ms-dos.                                   }
{ Copyright (C) 2005  Gorka Surez Garca                                        }
{                                                                                }
{ LogMovs.pas is part of "Ajedrez".                                              }
{                                                                                }
{ "Ajedrez" is free software; you can redistribute it and/or                     }
{ modify it under the terms of the GNU General Public License                    }
{ as published by the Free Software Foundation; either version 2                 }
{ of the License, or (at your option) any later version.                         }
{                                                                                }
{ "Ajedrez" is distributed in the hope that it will be useful,                   }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of                 }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  }
{ GNU General Public License for more details.                                   }
{                                                                                }
{ You should have received a copy of the GNU General Public License              }
{ along with "Ajedrez"; if not, write to the Free Software                       }
{ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.    }
{********************************************************************************}
unit LogMovs;
{********************************************************************************}
interface {parte publica}
uses Lesslib, Nucleo, crt;
{********************************************************************************}
{ Funciones para controlar la lgica de los movimientos del juego.               }
{********************************************************************************}
procedure ObtenerFicha      (var Nucleo: tNucleo; c, f: byte; var fic, col: byte);
function  DarNumFicha       (var Nucleo: tNucleo; color, c, f: byte): byte;
function  ValidarCasilla    (var Nucleo: tNucleo; color, c, f: byte): boolean;
function  HayJaque          (var Nucleo: tNucleo; color, c, f: byte): boolean;
function  HayJaqueMate      (var Nucleo: tNucleo): boolean;
function  ValidarMovimiento (var Nucleo: tNucleo; color, ficha, c, f: byte): boolean;
procedure DarMovimiento     (var Nucleo: tNucleo; mov: integer; turno, ficha: byte; var c, f: byte);
procedure CambiarTurno      (var Nucleo: tNucleo);


{********************************************************************************}
implementation {parte privada}
{********************************************************************************}


{********************************************************************************}
{ Funcin que cambia el turno del juego.                                         }
{********************************************************************************}
procedure CambiarTurno (var Nucleo: tNucleo);
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si alguno de los reyes est muerto, para dar la victoria.       }
   {-----------------------------------------------------------------------------}
   if Nucleo.Juego.Fichas[FICHA_BLANCA, REY].Tipo = FICHA_MUERTA then
      Nucleo.Juego.Victoria := VICTORIA_J2;

   if Nucleo.Juego.Fichas[FICHA_NEGRA, REY].Tipo = FICHA_MUERTA then
      Nucleo.Juego.Victoria := VICTORIA_J1;

   Nucleo.Juego.JaqueN := false;
   Nucleo.Juego.JaqueB := false;

   Nucleo.Juego.NoPeon := true;

   {-----------------------------------------------------------------------------}
   { Y cambiamos el turno del juego, comprobando si hay jaque o jaque mate.      }
   {-----------------------------------------------------------------------------}
   if Nucleo.Juego.Turno = TURNO_BLANCAS then
    begin
      Nucleo.Juego.Turno := TURNO_NEGRAS;

      if HayJaque(Nucleo, TURNO_NEGRAS, 0, 0) then begin
         if HayJaqueMate(Nucleo) then begin
            Nucleo.Juego.JaqueN  := true;
            Nucleo.Juego.Mensaje := MSJ_JAQUEMN;
         end else begin
            Nucleo.Juego.Mensaje := MSJ_JAQUEN;
         end;
      end else begin
         if HayJaqueMate(Nucleo) then begin
            Nucleo.Juego.Victoria := JUEGO_TABLAS;
         end else begin
            Nucleo.Juego.Mensaje := MSJ_NINGUNO;
         end;
      end;
    end
   else
    begin
      Nucleo.Juego.Turno := TURNO_BLANCAS;

      if HayJaque(Nucleo, TURNO_BLANCAS, 0, 0) then begin
         if HayJaqueMate(Nucleo) then begin
            Nucleo.Juego.JaqueB  := true;
            Nucleo.Juego.Mensaje := MSJ_JAQUEMB;
         end else begin
            Nucleo.Juego.Mensaje := MSJ_JAQUEB;
         end;
      end else begin
         if HayJaqueMate(Nucleo) then begin
            Nucleo.Juego.Victoria := JUEGO_TABLAS;
         end else begin
            Nucleo.Juego.Mensaje := MSJ_NINGUNO;
         end;
      end;
    end;

   Nucleo.Juego.NoPeon := false;
end;

{********************************************************************************}
{ Funcin para crear la lista actual de posibles movimientos.                    }
{********************************************************************************}
procedure DarMovimiento (var Nucleo: tNucleo; mov: integer; turno, ficha: byte;
                         var c, f: byte);
var
   incc, incf: shortint;
begin
   if (0 < ficha) and (ficha <= MAX_FICHAS_JUG) then begin
      {--------------------------------------------------------------------------}
      { Inicializamos los incrementos a esa cantidad barbara, por si se ha       }
      { metido un valor de mov erroneo, para el tipo de ficha indicado.          }
      {--------------------------------------------------------------------------}
      incc := 100;
      incf := 100;

      case Nucleo.Juego.Fichas[turno, ficha].Tipo of
         {-----------------------------------------------------------------------}
         { Devuelve los 4 movimientos posibles de los 8 peones del tablero.      }
         {-----------------------------------------------------------------------}
         FICHA_PEON: if (0 < mov) and (mov <= MAX_MOVS_PEON) then begin
            if turno = TURNO_BLANCAS then begin { Las fichas suben }
               incf := Nucleo.MovsFichas.PeonBlanco[mov].incf;
               incc := Nucleo.MovsFichas.PeonBlanco[mov].incc;
            end else begin { Las fichas bajan }
               incf := Nucleo.MovsFichas.PeonNegro[mov].incf;
               incc := Nucleo.MovsFichas.PeonNegro[mov].incc;
            end;
         end;

         {-----------------------------------------------------------------------}
         { Devuelve los 28 movimientos posibles de la torre.                     }
         {-----------------------------------------------------------------------}
         FICHA_TORRE: if (0 < mov) and (mov <= MAX_MOVS_TORRE) then begin
            incf := Nucleo.MovsFichas.Torre[mov].incf;
            incc := Nucleo.MovsFichas.Torre[mov].incc;
         end;

         {-----------------------------------------------------------------------}
         { Devuelve los 8 movimientos posibles del caballo.                      }
         {-----------------------------------------------------------------------}
         FICHA_CABALLO: if (0 < mov) and (mov <= MAX_MOVS_CABALLO) then begin
            incf := Nucleo.MovsFichas.Caballo[mov].incf;
            incc := Nucleo.MovsFichas.Caballo[mov].incc;
         end;

         {-----------------------------------------------------------------------}
         { Devuelve los 28 movimientos posibles del alfil.                       }
         {-----------------------------------------------------------------------}
         FICHA_ALFIL: if (0 < mov) and (mov <= MAX_MOVS_ALFIL) then begin
            incf := Nucleo.MovsFichas.Alfil[mov].incf;
            incc := Nucleo.MovsFichas.Alfil[mov].incc;
         end;

         {-----------------------------------------------------------------------}
         { Devuelve los 56 movimientos posibles de la reina.                     }
         {-----------------------------------------------------------------------}
         FICHA_REINA: if (0 < mov) and (mov <= MAX_MOVS_REINA) then begin
            incf := Nucleo.MovsFichas.Reina[mov].incf;
            incc := Nucleo.MovsFichas.Reina[mov].incc;
         end;

         {-----------------------------------------------------------------------}
         { Devuelve los 10 movimientos posibles del rey.                         }
         {-----------------------------------------------------------------------}
         FICHA_REY: if (0 < mov) and (mov <= MAX_MOVS_REY) then begin
            incf := Nucleo.MovsFichas.Rey[mov].incf;
            incc := Nucleo.MovsFichas.Rey[mov].incc;
         end;
      end;
      
      {--------------------------------------------------------------------------}
      { Comprobamos que ha sido seleccionada una ficha que no est muerta.       }
      { Finalmente ratificamos que las coordenadas no se salgan del tablero.     }
      {--------------------------------------------------------------------------}
      if Nucleo.Juego.Fichas[turno, ficha].Tipo <> FICHA_MUERTA then begin
         f := Nucleo.Juego.Fichas[turno, ficha].Fila + incf;
         c := Nucleo.Juego.Fichas[turno, ficha].Columna + incc;

         if not((0 < f) and (f < 9) and (0 < c) and (c < 9)) then begin
            f := 0;
            c := 0;
         end;
      end else begin
         f := 0;
         c := 0;
      end;
   end else begin
      f := 0;
      c := 0;
   end;
end;

{********************************************************************************}
{ Funcin que da la ficha que hay en una casilla.                                }
{********************************************************************************}
procedure ObtenerFicha (var Nucleo: tNucleo; c, f: byte; var fic, col: byte);
var
   ficha, color: byte;
   i, j: integer;
begin
   ficha := 0;
   color := 0;

   {-----------------------------------------------------------------------------}
   { Comprobamos pieza por pieza comprobando si coinciden las coordenadas que    }
   { tenemos con alguna de las coordenadas de las fichas.                        }
   {-----------------------------------------------------------------------------}
   for j := FICHA_BLANCA to FICHA_NEGRA do
      for i := 1 to MAX_FICHAS_JUG do
         if (Nucleo.Juego.Fichas[j, i].Fila = f) and
            (Nucleo.Juego.Fichas[j, i].Columna = c) and
            (Nucleo.Juego.Fichas[j, i].Tipo <> FICHA_MUERTA) then
          begin
            ficha := i;
            color := j;
          end;

   {-----------------------------------------------------------------------------}
   { Y devolvemos el resultado.                                                  }
   {-----------------------------------------------------------------------------}
   fic := ficha;
   col := color;
end;

{********************************************************************************}
{ Funcin que da la posicin de una ficha en el array de fichas del jugador.     }
{********************************************************************************}
function DarNumFicha (var Nucleo: tNucleo; color, c, f: byte): byte;
var
   ficha: byte;
   i: integer;
begin
   ficha := 0;

   {-----------------------------------------------------------------------------}
   { Comprobamos con el color, pieza por pieza si coinciden las coordenadas que  }
   { tenemos con alguna de las coordenadas de las fichas, de ese mismo color.    }
   {-----------------------------------------------------------------------------}
   for i := 1 to MAX_FICHAS_JUG do
      if (Nucleo.Juego.Fichas[color, i].Fila = f) and
         (Nucleo.Juego.Fichas[color, i].Columna = c) and
         (Nucleo.Juego.Fichas[color, i].Tipo <> FICHA_MUERTA) then
       begin
         ficha := i;
       end;

   {-----------------------------------------------------------------------------}
   { Y devolvemos el resultado.                                                  }
   {-----------------------------------------------------------------------------}
   DarNumFicha := ficha;
end;

{********************************************************************************}
{ Funcin que valida que la casilla pulsada corresponde a una pieza del bando.   }
{********************************************************************************}
function ValidarCasilla (var Nucleo: tNucleo; color, c, f: byte): boolean;
var
   resultado: boolean;
   i: integer;
begin
   resultado := false;

   {-----------------------------------------------------------------------------}
   { Comprobamos con el color, pieza por pieza si coinciden las coordenadas que  }
   { tenemos con alguna de las coordenadas de las fichas, de ese mismo color.    }
   {-----------------------------------------------------------------------------}
   for i := 1 to MAX_FICHAS_JUG do
      if (Nucleo.Juego.Fichas[color, i].Fila = f) and
         (Nucleo.Juego.Fichas[color, i].Columna = c) and
         (Nucleo.Juego.Fichas[color, i].Tipo <> FICHA_MUERTA) then
       begin
         resultado := true;
       end;

   {-----------------------------------------------------------------------------}
   { Y devolvemos el resultado.                                                  }
   {-----------------------------------------------------------------------------}
   ValidarCasilla := resultado;
end;

{********************************************************************************}
{ Funcin que determina si existe un jaque para el turno actual.                 }
{********************************************************************************}
function HayJaque (var Nucleo: tNucleo; color, c, f: byte): boolean;
var
   crey, frey: byte;
   fene, cene: byte;
   faux: tFicha;
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que no va a haber jaque.         }
   {-----------------------------------------------------------------------------}
   resultado := false;

   {-----------------------------------------------------------------------------}
   { Comprobamos que se este previamente en jaque mate.                          }
   {-----------------------------------------------------------------------------}
   if ((color = FICHA_BLANCA) and (Nucleo.Juego.JaqueB = false)) or
      ((color = FICHA_NEGRA)  and (Nucleo.Juego.JaqueN = false)) then
    begin
      {--------------------------------------------------------------------------}
      { Echamos un vistazo al color del enemigo.                                 }
      {--------------------------------------------------------------------------}
      if color = FICHA_BLANCA then
         cene := FICHA_NEGRA
      else if color = FICHA_NEGRA then
         cene := FICHA_BLANCA;

      crey := Nucleo.Juego.Fichas[color, REY].Columna;
      frey := Nucleo.Juego.Fichas[color, REY].Fila;

      {--------------------------------------------------------------------------}
      { Comprobamos con todas las fichas enemigas vivas, si pueden matar al rey, }
      { en la situacin actual del tablero.                                      }
      {--------------------------------------------------------------------------}
      for fene := 1 to MAX_FICHAS_JUG do
       begin
         if (Nucleo.Juego.Fichas[cene, fene].Tipo <> FICHA_MUERTA) then
          begin
            faux := Nucleo.Juego.Fichas[cene, fene];

            if (faux.Fila <> f) or (faux.Columna <> c) then
               {-----------------------------------------------------------------}
               { Si se da el caso de poder matar al rey, hay que restaurar el    }
               { estado anterior de esa ficha resucitar al rey del bando.        }
               {-----------------------------------------------------------------}
               if ValidarMovimiento(Nucleo, cene, fene, crey, frey) then
                begin
                  Nucleo.Juego.Fichas[color, REY].Tipo := FICHA_REY;
             
                  Nucleo.Juego.Fichas[cene, fene] := faux;

                  resultado := true;
                  break;
                end;
          end;
       end;

      if resultado then
         Nucleo.Juego.Mensaje := MSJ_MOVINVJ;
    end;

   {-----------------------------------------------------------------------------}
   { Y devolvemos el resultado, rezando para que todo haya ido bien.             }
   {-----------------------------------------------------------------------------}
   HayJaque := resultado;
end;

{********************************************************************************}
{ Funcin que determina si existe un jaque para el turno actual.                 }
{********************************************************************************}
function HayJaqueMate (var Nucleo: tNucleo): boolean;
var
   i, ficha, c, f: byte;
   faux: tFichas;
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber jaque mate.       }
   {-----------------------------------------------------------------------------}
   resultado := false;

   {-----------------------------------------------------------------------------}
   { Comprobamos a ver si se puede mover alguna ficha para evitar el jaque mate. }
   {-----------------------------------------------------------------------------}
   for ficha := 1 to MAX_FICHAS_JUG do begin
      for i := 1 to Nucleo.MovsFichas.NumMovs[Nucleo.Juego.Fichas[Nucleo.Juego.Turno, ficha].Tipo] do begin
         DarMovimiento(Nucleo, i, Nucleo.Juego.Turno, ficha, c, f);
         if (f > 0) and (c > 0) then begin
            faux := Nucleo.Juego.Fichas;

            resultado := ValidarMovimiento(Nucleo, Nucleo.Juego.Turno, ficha, c, f);

            Nucleo.Juego.Fichas := faux;

            if resultado then break;
         end;
      end;

      if resultado then break;
   end;

   {-----------------------------------------------------------------------------}
   { Y devolvemos el resultado, rezando para que todo haya ido bien.             }
   {-----------------------------------------------------------------------------}
   HayJaqueMate := not resultado;
end;

{********************************************************************************}
{ Funcin que elige una ficha para intercambiar con el peon al llegar al final.  }
{********************************************************************************}
procedure CambiarPeon (var Nucleo: tNucleo; color, ficha: byte);
var
   salir: boolean;
   letra: char;
begin
   salir := false;

   repeat
      FuenteEscribir('Elige ficha:',  216, 171, COL_TXTMSJ, Nucleo.Fuente);
      FuenteEscribir('T / C / A / R', 216, 185, COL_TXTMSJ, Nucleo.Fuente);
      M13hActualizarPantalla;
      letra := readkey;

      case letra of
         'T', 't':
          begin
            Nucleo.Juego.Fichas[color, ficha].Tipo := FICHA_TORRE;
            salir := true;
          end;
       
         'A', 'a':
          begin
            Nucleo.Juego.Fichas[color, ficha].Tipo := FICHA_ALFIL;
            salir := true;
          end;
       
         'C', 'c':
          begin
            Nucleo.Juego.Fichas[color, ficha].Tipo := FICHA_CABALLO;
            salir := true;
          end;
       
         'R', 'r':
          begin
            Nucleo.Juego.Fichas[color, ficha].Tipo := FICHA_REINA;
            salir := true;
          end;
      end;
   until salir;
end;

{********************************************************************************}
{ Funcin que valida el posible movimiento de un pen blanco.                    }
{********************************************************************************}
function ValidarMovPeonBlanco (var Nucleo: tNucleo; ficha, c, f: byte): boolean;
var
   auxc, auxf: byte;
   fene, cene: byte;
   fene2, cene2: byte;
   color: byte;
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;

   color := FICHA_BLANCA;

   {-----------------------------------------------------------------------------}
   { Comprobamos la primera alternativa lgica posible, que se quiere mover      }
   { hacia delante en la misma columna, dos casillas. Para ello solo podr hacer }
   { dicho movimiento desde la fila de inicio, que en el caso de las blancas se  }
   { trata de la fila nmero 2.                                                  }
   {-----------------------------------------------------------------------------}
   if (Nucleo.Juego.Fichas[color, ficha].Columna = c) and 
      (Nucleo.Juego.Fichas[color, ficha].Fila = 2) and 
      (Nucleo.Juego.Fichas[color, ficha].Fila = (f - 2))then
    begin
      ObtenerFicha(Nucleo, c, f, fene, cene);
      ObtenerFicha(Nucleo, c, f - 1, fene2, cene2);

      {--------------------------------------------------------------------------}
      { Miramos a ver si hay fichas en alguna de las dos casillas                }
      { siguientes, para comprobar que se puede realizar el movimiento.          }
      {--------------------------------------------------------------------------}
      if (fene = 0) and (cene = 0) and (fene2 = 0) and (cene2 = 0) then
       begin
         auxf := Nucleo.Juego.Fichas[color, ficha].Fila;
         auxc := Nucleo.Juego.Fichas[color, ficha].Columna;

         Nucleo.Juego.Fichas[color, ficha].Fila    := f;
         Nucleo.Juego.Fichas[color, ficha].Columna := c;

         {-----------------------------------------------------------------------}
         { Comprobamos si la situacin actual da la posibilidad de que se        }
         { de un jaque mate por accidente al mover el jugador.                   }
         {-----------------------------------------------------------------------}
         if HayJaque(Nucleo, color, c, f) then
          begin
            Nucleo.Juego.Fichas[color, ficha].Fila    := auxf;
            Nucleo.Juego.Fichas[color, ficha].Columna := auxc;
          end
         {-----------------------------------------------------------------------}
         { Si todo ha salido bien, comprobamos unos ltimos detalles.            }
         {-----------------------------------------------------------------------}
         else
          begin
            ObtenerFicha(Nucleo, c - 1, f, fene, cene);
            ObtenerFicha(Nucleo, c + 1, f, fene2, cene2);

            {--------------------------------------------------------------------}
            { Tras haber movido, hemos de comprobar si hay fichas a los          }
            { lados, para activarles el indicador de que pueden atacarnos.       }
            {--------------------------------------------------------------------}
            if (0 < fene) and (fene <= MAX_PEONES) and (cene = FICHA_NEGRA) and
               (Nucleo.Juego.Fichas[cene, fene].Tipo = FICHA_PEON) then
             begin
               Nucleo.Juego.Fichas[cene, fene].Movesp   := MOVESP_PEON_A;
               Nucleo.Juego.Fichas[color, ficha].Movesp := MOVESP_PEON_V;
             end;

            if (0 < fene2) and (fene2 <= MAX_PEONES) and (cene2 = FICHA_NEGRA) and
               (Nucleo.Juego.Fichas[cene2, fene2].Tipo = FICHA_PEON) then
             begin
               Nucleo.Juego.Fichas[cene2, fene2].Movesp := MOVESP_PEON_A;
               Nucleo.Juego.Fichas[color, ficha].Movesp := MOVESP_PEON_V;
             end;

            {--------------------------------------------------------------------}
            { Si todo ha salido bien, devolvemos true.                           }
            {--------------------------------------------------------------------}
            resultado := true;
          end;
       end;
    end
   {-----------------------------------------------------------------------------}
   { La segunda alternativa lgica posible es que se quiere mover hacia delante  }
   { tan solo una sola casilla. Teniendo en cuenta lo que pasara de llegar al   }
   { final del tablero con el pen (la transformacin en saiya-jin).             }
   {-----------------------------------------------------------------------------}
   else if (Nucleo.Juego.Fichas[color, ficha].Columna = c) and 
           (Nucleo.Juego.Fichas[color, ficha].Fila = (f - 1))then
    begin
      ObtenerFicha(Nucleo, c, f, fene, cene);

      {--------------------------------------------------------------------------}
      { Echamos un vistazo a ver si hay alguna ficha en esa casilla, ya que solo }
      { podr moverse el peon en caso de que no haya ninguna ficha en la casilla }
      { a la que se piensa mover.                                                }
      {--------------------------------------------------------------------------}
      if (fene = 0) and (cene = 0) then
       begin
         auxf := Nucleo.Juego.Fichas[color, ficha].Fila;
         auxc := Nucleo.Juego.Fichas[color, ficha].Columna;

         Nucleo.Juego.Fichas[color, ficha].Fila    := f;
         Nucleo.Juego.Fichas[color, ficha].Columna := c;

         {-----------------------------------------------------------------------}
         { Comprobamos si la situacin actual da la posibilidad de que se de un  }
         { jaque mate por accidente al mover el jugador.                         }
         {-----------------------------------------------------------------------}
         if HayJaque(Nucleo, color, c, f) then
          begin
            Nucleo.Juego.Fichas[color, ficha].Fila    := auxf;
            Nucleo.Juego.Fichas[color, ficha].Columna := auxc;
          end
         {-----------------------------------------------------------------------}
         { Si todo ha salido bien, comprobamos unos ltimos detalles.            }
         {-----------------------------------------------------------------------}
         else
          begin
            {--------------------------------------------------------------------}
            { Aunque no sin antes comprobar si el pen ha llegado al final del   }
            { tablero, dndole la opcin de convertirse en saiya-jin.            }
            {--------------------------------------------------------------------}
            if (f = 8) then
               if (Nucleo.Juego.NoPeon = false) then
                  CambiarPeon(Nucleo, color, ficha)
               else
                  Nucleo.Juego.Fichas[color, ficha].Tipo := FICHA_REINA;

            {--------------------------------------------------------------------}
            { Si todo ha salido bien, devolvemos true.                           }
            {--------------------------------------------------------------------}
            resultado := true;
          end;
       end;
    end
   {-----------------------------------------------------------------------------}
   { Nuestra tercera alternativa es atacar hacia la derecha o la izquierda.      }
   {-----------------------------------------------------------------------------}
   else if ((Nucleo.Juego.Fichas[color, ficha].Columna = (c - 1)) or
           (Nucleo.Juego.Fichas[color, ficha].Columna = (c + 1))) and 
           (Nucleo.Juego.Fichas[color, ficha].Fila = (f - 1)) then
    begin
      {--------------------------------------------------------------------------}
      { Obtenemos el contenido de la casilla a la que pensamos ir, y de la que   }
      { est debajo de esta, para ver si hay un peon enemigo.                    }
      {--------------------------------------------------------------------------}
      ObtenerFicha(Nucleo, c, f, fene, cene);
      ObtenerFicha(Nucleo, c, f - 1, fene2, cene2);
      
      {--------------------------------------------------------------------------}
      { Echamos un vistazo a ver si hay alguna ficha en esa casilla, ya que solo }
      { podemos movernos para atacar si hay una ficha del bando contrario en esa }
      { casilla que pensamos invadir.                                            }
      {--------------------------------------------------------------------------}
      if (fene <> 0) and (cene = FICHA_NEGRA) then
       begin
         auxf := Nucleo.Juego.Fichas[color, ficha].Fila;
         auxc := Nucleo.Juego.Fichas[color, ficha].Columna;

         Nucleo.Juego.Fichas[color, ficha].Fila    := f;
         Nucleo.Juego.Fichas[color, ficha].Columna := c;

         {-----------------------------------------------------------------------}
         { Comprobamos si la situacin actual da la posibilidad de que se de un  }
         { jaque mate por accidente al mover el jugador.                         }
         {-----------------------------------------------------------------------}
         if HayJaque(Nucleo, color, c, f) and (fene <> REY) then
          begin
            Nucleo.Juego.Fichas[color, ficha].Fila    := auxf;
            Nucleo.Juego.Fichas[color, ficha].Columna := auxc;
          end
         {-----------------------------------------------------------------------}
         { Si todo ha salido bien, matamos a la ficha enemiga.                   }
         {-----------------------------------------------------------------------}
         else
          begin
            Nucleo.Juego.Fichas[cene, fene].Tipo := FICHA_MUERTA;

            {--------------------------------------------------------------------}
            { Aunque no sin antes comprobar si el pen ha llegado al final del   }
            { tablero, dndole la opcin de convertirse en saiya-jin.            }
            {--------------------------------------------------------------------}
            if (f = 8) then
               if (Nucleo.Juego.NoPeon = false) then
                  CambiarPeon(Nucleo, color, ficha)
               else
                  Nucleo.Juego.Fichas[color, ficha].Tipo := FICHA_REINA;

            {--------------------------------------------------------------------}
            { Si todo ha salido bien, devolvemos true.                           }
            {--------------------------------------------------------------------}
            resultado := true;
          end;
       end
      {--------------------------------------------------------------------------}
      { Pero tambin existe otra posibilidad de ataque, que estemos en la fila 5 }
      { y acabe un peon enemigo de avanzar dos casillas, con el fin de evitar    }
      { que nos lo queramos comer.                                               }
      {--------------------------------------------------------------------------}
      else if (Nucleo.Juego.Fichas[color, ficha].Fila = 5) and
              (0 < fene2) and (fene2 <= MAX_PEONES) and (cene2 = FICHA_NEGRA) and
              (Nucleo.Juego.Fichas[cene2, fene2].Tipo = FICHA_PEON) and
              (Nucleo.Juego.Fichas[cene2, fene2].Movesp = MOVESP_PEON_V) and
              (Nucleo.Juego.Fichas[color, ficha].Movesp = MOVESP_PEON_A) then
       begin
         auxf := Nucleo.Juego.Fichas[color, ficha].Fila;
         auxc := Nucleo.Juego.Fichas[color, ficha].Columna;

         Nucleo.Juego.Fichas[color, ficha].Fila    := f;
         Nucleo.Juego.Fichas[color, ficha].Columna := c;

         {-----------------------------------------------------------------------}
         { Comprobamos si la situacin actual da la posibilidad de que se de un  }
         { jaque mate por accidente al mover el jugador.                         }
         {-----------------------------------------------------------------------}
         if HayJaque(Nucleo, color, c, f) then
          begin
            Nucleo.Juego.Fichas[color, ficha].Fila    := auxf;
            Nucleo.Juego.Fichas[color, ficha].Columna := auxc;
          end
         {-----------------------------------------------------------------------}
         { Si todo ha salido bien, matamos a la ficha enemiga.                   }
         {-----------------------------------------------------------------------}
         else
          begin
            Nucleo.Juego.Fichas[cene2, fene2].Tipo := FICHA_MUERTA;
            resultado := true;
          end;
       end;
    end;

   {-----------------------------------------------------------------------------}
   { Y devolvemos el resultado, rezando para que todo haya ido bien.             }
   {-----------------------------------------------------------------------------}
   ValidarMovPeonBlanco := resultado;
end;

{********************************************************************************}
{ Funcin que valida el posible movimiento de un pen negro.                     }
{********************************************************************************}
function ValidarMovPeonNegro (var Nucleo: tNucleo; ficha, c, f: byte): boolean;
var
   auxc, auxf: byte;
   fene, cene: byte;
   fene2, cene2: byte;
   color: byte;
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;

   color := FICHA_NEGRA;

   {-----------------------------------------------------------------------------}
   { Comprobamos la primera alternativa lgica posible, que se quiere mover      }
   { hacia delante en la misma columna, dos casillas. Para ello solo podr hacer }
   { dicho movimiento desde la fila de inicio, que en el caso de las negras se   }
   { trata de la fila nmero 7.                                                  }
   {-----------------------------------------------------------------------------}
   if (Nucleo.Juego.Fichas[color, ficha].Columna = c) and 
      (Nucleo.Juego.Fichas[color, ficha].Fila = 7) and 
      (Nucleo.Juego.Fichas[color, ficha].Fila = (f + 2))then
    begin
      ObtenerFicha(Nucleo, c, f, fene, cene);
      ObtenerFicha(Nucleo, c, f + 1, fene2, cene2);

      {--------------------------------------------------------------------------}
      { Miramos a ver si hay fichas en alguna de las dos casillas                }
      { siguientes, para comprobar que se puede realizar el movimiento.          }
      {--------------------------------------------------------------------------}
      if (fene = 0) and (cene = 0) and (fene2 = 0) and (cene2 = 0) then
       begin
         auxf := Nucleo.Juego.Fichas[color, ficha].Fila;
         auxc := Nucleo.Juego.Fichas[color, ficha].Columna;

         Nucleo.Juego.Fichas[color, ficha].Fila    := f;
         Nucleo.Juego.Fichas[color, ficha].Columna := c;

         {-----------------------------------------------------------------------}
         { Comprobamos si la situacin actual da la posibilidad de que se        }
         { de un jaque mate por accidente al mover el jugador.                   }
         {-----------------------------------------------------------------------}
         if HayJaque(Nucleo, color, c, f) then
          begin
            Nucleo.Juego.Fichas[color, ficha].Fila    := auxf;
            Nucleo.Juego.Fichas[color, ficha].Columna := auxc;
          end
         {-----------------------------------------------------------------------}
         { Si todo ha salido bien, comprobamos unos ltimos detalles.            }
         {-----------------------------------------------------------------------}
         else
          begin
            ObtenerFicha(Nucleo, c - 1, f, fene, cene);
            ObtenerFicha(Nucleo, c + 1, f, fene2, cene2);

            {--------------------------------------------------------------------}
            { Tras haber movido, hemos de comprobar si hay fichas a los          }
            { lados, para activarles el indicador de que pueden atacarnos.       }
            {--------------------------------------------------------------------}
            if (0 < fene) and (fene <= MAX_PEONES) and (cene = FICHA_BLANCA) and
               (Nucleo.Juego.Fichas[cene, fene].Tipo = FICHA_PEON) then
             begin
               Nucleo.Juego.Fichas[cene, fene].Movesp   := MOVESP_PEON_A;
               Nucleo.Juego.Fichas[color, ficha].Movesp := MOVESP_PEON_V;
             end;

            if (0 < fene2) and (fene2 <= MAX_PEONES) and (cene2 = FICHA_BLANCA) and
               (Nucleo.Juego.Fichas[cene2, fene2].Tipo = FICHA_PEON) then
             begin
               Nucleo.Juego.Fichas[cene2, fene2].Movesp := MOVESP_PEON_A;
               Nucleo.Juego.Fichas[color, ficha].Movesp := MOVESP_PEON_V;
             end;

            {--------------------------------------------------------------------}
            { Si todo ha salido bien, devolvemos true.                           }
            {--------------------------------------------------------------------}
            resultado := true;
          end;
       end;
    end
   {-----------------------------------------------------------------------------}
   { La segunda alternativa lgica posible es que se quiere mover hacia delante  }
   { tan solo una sola casilla. Teniendo en cuenta lo que pasara de llegar al   }
   { final del tablero con el pen (la transformacin en saiya-jin).             }
   {-----------------------------------------------------------------------------}
   else if (Nucleo.Juego.Fichas[color, ficha].Columna = c) and 
           (Nucleo.Juego.Fichas[color, ficha].Fila = (f + 1))then
    begin
      ObtenerFicha(Nucleo, c, f, fene, cene);

      {--------------------------------------------------------------------------}
      { Echamos un vistazo a ver si hay alguna ficha en esa casilla, ya que solo }
      { podr moverse el peon en caso de que no haya ninguna ficha en la casilla }
      { a la que se piensa mover.                                                }
      {--------------------------------------------------------------------------}
      if (fene = 0) and (cene = 0) then
       begin
         auxf := Nucleo.Juego.Fichas[color, ficha].Fila;
         auxc := Nucleo.Juego.Fichas[color, ficha].Columna;

         Nucleo.Juego.Fichas[color, ficha].Fila    := f;
         Nucleo.Juego.Fichas[color, ficha].Columna := c;

         {-----------------------------------------------------------------------}
         { Comprobamos si la situacin actual da la posibilidad de que se de un  }
         { jaque mate por accidente al mover el jugador.                         }
         {-----------------------------------------------------------------------}
         if HayJaque(Nucleo, color, c, f) then
          begin
            Nucleo.Juego.Fichas[color, ficha].Fila    := auxf;
            Nucleo.Juego.Fichas[color, ficha].Columna := auxc;
          end
         {-----------------------------------------------------------------------}
         { Si todo ha salido bien, comprobamos unos ltimos detalles.            }
         {-----------------------------------------------------------------------}
         else
          begin
            {--------------------------------------------------------------------}
            { Aunque no sin antes comprobar si el pen ha llegado al final del   }
            { tablero, dndole la opcin de convertirse en saiya-jin.            }
            {--------------------------------------------------------------------}
            if (f = 1) then
               if (Nucleo.Juego.NoPeon = false) then
                  CambiarPeon(Nucleo, color, ficha)
               else
                  Nucleo.Juego.Fichas[color, ficha].Tipo := FICHA_REINA;

            {--------------------------------------------------------------------}
            { Si todo ha salido bien, devolvemos true.                           }
            {--------------------------------------------------------------------}
            resultado := true;
          end;
       end;
    end
   {-----------------------------------------------------------------------------}
   { Nuestra tercera alternativa es atacar hacia la derecha o la izquierda.      }
   {-----------------------------------------------------------------------------}
   else if ((Nucleo.Juego.Fichas[color, ficha].Columna = (c - 1)) or
           (Nucleo.Juego.Fichas[color, ficha].Columna = (c + 1))) and 
           (Nucleo.Juego.Fichas[color, ficha].Fila = (f + 1)) then
    begin
      {--------------------------------------------------------------------------}
      { Obtenemos el contenido de la casilla a la que pensamos ir, y de la que   }
      { est debajo de esta, para ver si hay un peon enemigo.                    }
      {--------------------------------------------------------------------------}
      ObtenerFicha(Nucleo, c, f, fene, cene);
      ObtenerFicha(Nucleo, c, f + 1, fene2, cene2);
      
      {--------------------------------------------------------------------------}
      { Echamos un vistazo a ver si hay alguna ficha en esa casilla, ya que solo }
      { podemos movernos para atacar si hay una ficha del bando contrario en esa }
      { casilla que pensamos invadir.                                            }
      {--------------------------------------------------------------------------}
      if (fene <> 0) and (cene = FICHA_BLANCA) then
       begin
         auxf := Nucleo.Juego.Fichas[color, ficha].Fila;
         auxc := Nucleo.Juego.Fichas[color, ficha].Columna;

         Nucleo.Juego.Fichas[color, ficha].Fila    := f;
         Nucleo.Juego.Fichas[color, ficha].Columna := c;

         {-----------------------------------------------------------------------}
         { Comprobamos si la situacin actual da la posibilidad de que se de un  }
         { jaque mate por accidente al mover el jugador.                         }
         {-----------------------------------------------------------------------}
         if HayJaque(Nucleo, color, c, f) and (fene <> REY) then
          begin
            Nucleo.Juego.Fichas[color, ficha].Fila    := auxf;
            Nucleo.Juego.Fichas[color, ficha].Columna := auxc;
          end
         {-----------------------------------------------------------------------}
         { Si todo ha salido bien, matamos a la ficha enemiga.                   }
         {-----------------------------------------------------------------------}
         else
          begin
            Nucleo.Juego.Fichas[cene, fene].Tipo := FICHA_MUERTA;

            {--------------------------------------------------------------------}
            { Aunque no sin antes comprobar si el pen ha llegado al final del   }
            { tablero, dndole la opcin de convertirse en saiya-jin.            }
            {--------------------------------------------------------------------}
            if (f = 1) then
               if (Nucleo.Juego.NoPeon = false) then
                  CambiarPeon(Nucleo, color, ficha)
               else
                  Nucleo.Juego.Fichas[color, ficha].Tipo := FICHA_REINA;

            {--------------------------------------------------------------------}
            { Si todo ha salido bien, devolvemos true.                           }
            {--------------------------------------------------------------------}
            resultado := true;
          end;
       end
      {--------------------------------------------------------------------------}
      { Pero tambin existe otra posibilidad de ataque, que estemos en la fila 4 }
      { y acabe un peon enemigo de avanzar dos casillas, con el fin de evitar    }
      { que nos lo queramos comer.                                               }
      {--------------------------------------------------------------------------}
      else if (Nucleo.Juego.Fichas[color, ficha].Fila = 4) and
              (0 < fene2) and (fene2 <= MAX_PEONES) and (cene2 = FICHA_BLANCA) and
              (Nucleo.Juego.Fichas[cene2, fene2].Tipo = FICHA_PEON) and
              (Nucleo.Juego.Fichas[cene2, fene2].Movesp = MOVESP_PEON_V) and
              (Nucleo.Juego.Fichas[color, ficha].Movesp = MOVESP_PEON_A) then
       begin
         auxf := Nucleo.Juego.Fichas[color, ficha].Fila;
         auxc := Nucleo.Juego.Fichas[color, ficha].Columna;

         Nucleo.Juego.Fichas[color, ficha].Fila    := f;
         Nucleo.Juego.Fichas[color, ficha].Columna := c;

         {-----------------------------------------------------------------------}
         { Comprobamos si la situacin actual da la posibilidad de que se de un  }
         { jaque mate por accidente al mover el jugador.                         }
         {-----------------------------------------------------------------------}
         if HayJaque(Nucleo, color, c, f) then
          begin
            Nucleo.Juego.Fichas[color, ficha].Fila    := auxf;
            Nucleo.Juego.Fichas[color, ficha].Columna := auxc;
          end
         {-----------------------------------------------------------------------}
         { Si todo ha salido bien, matamos a la ficha enemiga.                   }
         {-----------------------------------------------------------------------}
         else
          begin
            Nucleo.Juego.Fichas[cene2, fene2].Tipo := FICHA_MUERTA;
            resultado := true;
          end;
       end;
    end;

   {-----------------------------------------------------------------------------}
   { Y devolvemos el resultado, rezando para que todo haya ido bien.             }
   {-----------------------------------------------------------------------------}
   ValidarMovPeonNegro := resultado;
end;

{********************************************************************************}
{ Funcin que valida el posible movimiento de un pen.                           }
{********************************************************************************}
function ValidarMovPeon (var Nucleo: tNucleo; color, ficha, c, f: byte): boolean;
var
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;

   {-----------------------------------------------------------------------------}
   { Miramos a ver si el peon es de las fichas blancas.                          }
   {-----------------------------------------------------------------------------}
   if color = FICHA_BLANCA then
      resultado := ValidarMovPeonBlanco(Nucleo, ficha, c, f)
   {-----------------------------------------------------------------------------}
   { Si no es blanco entonces es negro, un poco obvio, no?                      }
   {-----------------------------------------------------------------------------}
   else
      resultado := ValidarMovPeonNegro(Nucleo, ficha, c, f);

   {-----------------------------------------------------------------------------}
   { Y devolvemos el resultado, rezando para que todo haya ido bien.             }
   {-----------------------------------------------------------------------------}
   ValidarMovPeon := resultado;
end;

{********************************************************************************}
{ Calcula si hay alguna pieza en la trayectoria.                                 }
{********************************************************************************}
function CalcularColision (var Nucleo: tNucleo; ci, fi, cf, ff: byte): boolean;
var
   resultado: boolean;
   i, j: byte;
   ficha, color: byte;
begin
   resultado := false;

   {-----------------------------------------------------------------------------}
   { Se trata de un movimiento por la misma columna, por ello recorremos las     }
   { casillas intermedias de dicha columna, para ver si hay alguna ficha.        }
   {-----------------------------------------------------------------------------}
   if ci = cf then
    begin
      i := fi;
      
      if i < ff then
         i := i + 1
      else if i > ff then
         i := i - 1;

      while (i <> ff) and (not resultado) do
       begin
         ObtenerFicha(Nucleo, cf, i, ficha, color);

         if ficha <> 0 then
            resultado := true;
            
         if i < ff then
            i := i + 1
         else if i > ff then
            i := i - 1;
       end;
    end
   {-----------------------------------------------------------------------------}
   { Se trata de un movimiento por la misma fila, por ello recorremos las        }
   { casillas intermedias de dicha fila, para ver si hay alguna ficha.           }
   {-----------------------------------------------------------------------------}
   else if fi = ff then
    begin
      i := ci;
      
      if i < cf then
         i := i + 1
      else if i > cf then
         i := i - 1;

      while (i <> cf) and (not resultado) do
       begin
         ObtenerFicha(Nucleo, i, ff, ficha, color);

         if ficha <> 0 then
            resultado := true;
            
         if i < cf then
            i := i + 1
         else if i > cf then
            i := i - 1;
       end;
    end
   {-----------------------------------------------------------------------------}
   { Se trata de un movimiento por una diagonal, por ello recorremos las         }
   { casillas intermedias de dicha diagonal, desde el posterior a (ci, fi) hasta }
   { el predecesor de (c, f).                                                    }
   {-----------------------------------------------------------------------------}
   else if abs(fi - ff) = abs(ci - cf) then
    begin
      i := fi;
      j := ci;
      
      {--------------------------------------------------------------------------}
      { Avanzamos una casilla en la diagonal, para ello hemos de usar este       }
      { sistema. Dado que fi es distinto de f, y ci distinto de c, ya que de     }
      { haber sido iguales alguno de estos pares, se habra quedado en alguna de }
      { las secciones anteriores. Por ello aqu lo que se busca es ir desde      }
      { (ci, fi) hasta (c, f). Por ello si el contador i es menor que f, lo      }
      { incrementamos, pero si es mayor que f, lo decrementamos. Y lo mismo      }
      { haramos con el contador j, con respecto la coordenada c.                }
      {--------------------------------------------------------------------------}
      if i < ff then
         i := i + 1
      else if i > ff then
         i := i - 1;
         
      if j < cf then
         j := j + 1
      else if j > cf then
         j := j - 1;

      {--------------------------------------------------------------------------}
      { As que mientras no lleguemos a (c, f), comprobaremos la casilla.        }
      {--------------------------------------------------------------------------}
      while (i <> ff) and (j <> cf) and (not resultado) do
       begin
         ObtenerFicha(Nucleo, j, i, ficha, color);

         if ficha <> 0 then
            resultado := true;

         if i < ff then
            i := i + 1
         else if i > ff then
            i := i - 1;
            
         if j < cf then
            j := j + 1
         else if j > cf then
            j := j - 1;
       end;
    end;

   CalcularColision := resultado;
end;

{********************************************************************************}
{ Funcin que valida el posible movimiento de una torre.                         }
{********************************************************************************}
function ValidarMovTorre (var Nucleo: tNucleo; color, ficha, c, f: byte): boolean;
var
   auxc, auxf: byte;
   fene, cene: byte;
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;

   {-----------------------------------------------------------------------------}
   { Comprobamos que se mueve por la misma columna o por la misma fila.          }
   {-----------------------------------------------------------------------------}
   if (Nucleo.Juego.Fichas[color, ficha].Columna = c) or
      (Nucleo.Juego.Fichas[color, ficha].Fila    = f) then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos que no hay ninguna ficha intermedia.                         }
      {--------------------------------------------------------------------------}
      if not CalcularColision(Nucleo, Nucleo.Juego.Fichas[color, ficha].Columna,
         Nucleo.Juego.Fichas[color, ficha].Fila, c, f) then
       begin
         ObtenerFicha(Nucleo, c, f, fene, cene);

         auxf := Nucleo.Juego.Fichas[color, ficha].Fila;
         auxc := Nucleo.Juego.Fichas[color, ficha].Columna;

         Nucleo.Juego.Fichas[color, ficha].Fila    := f;
         Nucleo.Juego.Fichas[color, ficha].Columna := c;

         {-----------------------------------------------------------------------}
         { Comprobamos si la situacin actual da la posibilidad de que se de un  }
         { jaque mate por accidente al mover el jugador.                         }
         {-----------------------------------------------------------------------}
         if HayJaque(Nucleo, color, c, f) and (fene <> REY) then
          begin
            Nucleo.Juego.Fichas[color, ficha].Fila    := auxf;
            Nucleo.Juego.Fichas[color, ficha].Columna := auxc;
          end
         {-----------------------------------------------------------------------}
         { Si todo ha salido bien, comprobamos si hemos matado alguien.          }
         {-----------------------------------------------------------------------}
         else
          begin
            {--------------------------------------------------------------------}
            { Nos encontramos ante una pieza enemiga.                            }
            {--------------------------------------------------------------------}
            if (color <> cene) and (cene <> 0) then
             begin

               Nucleo.Juego.Fichas[cene, fene].Tipo := FICHA_MUERTA;
               resultado := true;
             end
            {--------------------------------------------------------------------}
            { Nos encontramos con una casilla vaca.                             }
            {--------------------------------------------------------------------}
            else if (cene = 0) then
             begin
               resultado := true;
             end;
          end;
       end;
    end;

   {-----------------------------------------------------------------------------}
   { Activamos el indicador que imposibilita que se ejecute el enroque.          }
   {-----------------------------------------------------------------------------}
   if (Nucleo.Juego.Fichas[color, ficha].Tipo = FICHA_TORRE) and resultado then
      Nucleo.Juego.Fichas[color, ficha].Movesp := MOVESP_ENROQUE;

   {-----------------------------------------------------------------------------}
   { Y devolvemos el resultado, rezando para que todo haya ido bien.             }
   {-----------------------------------------------------------------------------}
   ValidarMovTorre := resultado;
end;

{********************************************************************************}
{ Funcin que valida el posible movimiento de un caballo.                        }
{********************************************************************************}
function ValidarMovCaballo (var Nucleo: tNucleo; color, ficha, c, f: byte): boolean;
var
   auxc, auxf: byte;
   fene, cene: byte;
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;

   {-----------------------------------------------------------------------------}
   { Comprobamos que se de alguno de los 8 movimientos posibles del caballo.     }
   {-----------------------------------------------------------------------------}
   if ((Nucleo.Juego.Fichas[color, ficha].Columna = (c - 2)) and (Nucleo.Juego.Fichas[color, ficha].Fila = (f - 1))) or
      ((Nucleo.Juego.Fichas[color, ficha].Columna = (c - 2)) and (Nucleo.Juego.Fichas[color, ficha].Fila = (f + 1))) or
      ((Nucleo.Juego.Fichas[color, ficha].Columna = (c + 2)) and (Nucleo.Juego.Fichas[color, ficha].Fila = (f - 1))) or
      ((Nucleo.Juego.Fichas[color, ficha].Columna = (c + 2)) and (Nucleo.Juego.Fichas[color, ficha].Fila = (f + 1))) or
      ((Nucleo.Juego.Fichas[color, ficha].Columna = (c - 1)) and (Nucleo.Juego.Fichas[color, ficha].Fila = (f - 2))) or
      ((Nucleo.Juego.Fichas[color, ficha].Columna = (c - 1)) and (Nucleo.Juego.Fichas[color, ficha].Fila = (f + 2))) or
      ((Nucleo.Juego.Fichas[color, ficha].Columna = (c + 1)) and (Nucleo.Juego.Fichas[color, ficha].Fila = (f - 2))) or
      ((Nucleo.Juego.Fichas[color, ficha].Columna = (c + 1)) and (Nucleo.Juego.Fichas[color, ficha].Fila = (f + 2))) then
    begin
      ObtenerFicha(Nucleo, c, f, fene, cene);

      auxf := Nucleo.Juego.Fichas[color, ficha].Fila;
      auxc := Nucleo.Juego.Fichas[color, ficha].Columna;

      Nucleo.Juego.Fichas[color, ficha].Fila    := f;
      Nucleo.Juego.Fichas[color, ficha].Columna := c;

      {--------------------------------------------------------------------------}
      { Comprobamos si la situacin actual da la posibilidad de que se de un     }
      { jaque mate por accidente al mover el jugador.                            }
      {--------------------------------------------------------------------------}
      if HayJaque(Nucleo, color, c, f) and (fene <> REY) then
       begin
         Nucleo.Juego.Fichas[color, ficha].Fila    := auxf;
         Nucleo.Juego.Fichas[color, ficha].Columna := auxc;
       end
      {--------------------------------------------------------------------------}
      { Si todo ha salido bien, comprobamos si hemos matado alguien.             }
      {--------------------------------------------------------------------------}
      else
       begin
         {-----------------------------------------------------------------------}
         { Nos encontramos ante una pieza enemiga.                               }
         {-----------------------------------------------------------------------}
         if (color <> cene) and (cene <> 0) then
          begin
            Nucleo.Juego.Fichas[cene, fene].Tipo := FICHA_MUERTA;
            resultado := true;
          end
         {-----------------------------------------------------------------------}
         { Nos encontramos con una casilla vaca.                                }
         {-----------------------------------------------------------------------}
         else if (cene = 0) then
          begin
            resultado := true;
          end;
       end;
    end;

   {-----------------------------------------------------------------------------}
   { Y devolvemos el resultado, rezando para que todo haya ido bien.             }
   {-----------------------------------------------------------------------------}
   ValidarMovCaballo := resultado;
end;

{********************************************************************************}
{ Funcin que valida el posible movimiento de un alfil.                          }
{********************************************************************************}
function ValidarMovAlfil (var Nucleo: tNucleo; color, ficha, c, f: byte): boolean;
var
   auxc, auxf: byte;
   fene, cene: byte;
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;

   {-----------------------------------------------------------------------------}
   { Comprobamos que se mueve por la misma columna o por la misma fila.          }
   {-----------------------------------------------------------------------------}
   if abs(Nucleo.Juego.Fichas[color, ficha].Columna - c) =
      abs(Nucleo.Juego.Fichas[color, ficha].Fila    - f) then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos que no hay ninguna ficha intermedia.                         }
      {--------------------------------------------------------------------------}
      if not CalcularColision(Nucleo, Nucleo.Juego.Fichas[color, ficha].Columna,
         Nucleo.Juego.Fichas[color, ficha].Fila, c, f) then
       begin
         ObtenerFicha(Nucleo, c, f, fene, cene);

         auxf := Nucleo.Juego.Fichas[color, ficha].Fila;
         auxc := Nucleo.Juego.Fichas[color, ficha].Columna;

         Nucleo.Juego.Fichas[color, ficha].Fila    := f;
         Nucleo.Juego.Fichas[color, ficha].Columna := c;

         {-----------------------------------------------------------------------}
         { Comprobamos si la situacin actual da la posibilidad de que se de un  }
         { jaque mate por accidente al mover el jugador.                         }
         {-----------------------------------------------------------------------}
         if HayJaque(Nucleo, color, c, f) and (fene <> REY) then
          begin
            Nucleo.Juego.Fichas[color, ficha].Fila    := auxf;
            Nucleo.Juego.Fichas[color, ficha].Columna := auxc;
          end
         {-----------------------------------------------------------------------}
         { Si todo ha salido bien, comprobamos si hemos matado alguien.          }
         {-----------------------------------------------------------------------}
         else
          begin
            {--------------------------------------------------------------------}
            { Nos encontramos ante una pieza enemiga.                            }
            {--------------------------------------------------------------------}
            if (color <> cene) and (cene <> 0) then
             begin

               Nucleo.Juego.Fichas[cene, fene].Tipo := FICHA_MUERTA;
               resultado := true;
             end
            {--------------------------------------------------------------------}
            { Nos encontramos con una casilla vaca.                             }
            {--------------------------------------------------------------------}
            else if (cene = 0) then
             begin
               resultado := true;
             end;
          end;
       end;
    end;

   {-----------------------------------------------------------------------------}
   { Y devolvemos el resultado, rezando para que todo haya ido bien.             }
   {-----------------------------------------------------------------------------}
   ValidarMovAlfil := resultado;
end;

{********************************************************************************}
{ Funcin que valida el posible movimiento de una reina.                         }
{********************************************************************************}
function ValidarMovReina (var Nucleo: tNucleo; color, ficha, c, f: byte): boolean;
var
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;

   {-----------------------------------------------------------------------------}
   { Casualmente la reina se mueve como la torre o como el alfil.                }
   {-----------------------------------------------------------------------------}
   resultado := (ValidarMovTorre(Nucleo, color, ficha, c, f)) or 
                (ValidarMovAlfil(Nucleo, color, ficha, c, f));

   {-----------------------------------------------------------------------------}
   { Y devolvemos el resultado, rezando para que todo haya ido bien.             }
   {-----------------------------------------------------------------------------}
   ValidarMovReina := resultado;
end;

{********************************************************************************}
{ Funcin que valida el posible enroque de un rey.                               }
{********************************************************************************}
function ValidarEnroqueRey (var Nucleo: tNucleo; color, ficha, c, f: byte): boolean;
var
   auxc, auxf: byte;
   fene, cene: byte;
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;

   {-----------------------------------------------------------------------------}
   { Comprobamos que no estamos en jaque para poder mover.                       }
   {-----------------------------------------------------------------------------}
   if ((Nucleo.Juego.JaqueB = false) and (color = FICHA_BLANCA)) or
      ((Nucleo.Juego.JaqueN = false) and (color = FICHA_NEGRA)) then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos que no hay ninguna ficha intermedia, y as de paso           }
      { descubrimos si hay alguna ficha en la casilla final de la torre.         }
      {--------------------------------------------------------------------------}
      if not CalcularColision(Nucleo, Nucleo.Juego.Fichas[color, ficha].Columna,
         Nucleo.Juego.Fichas[color, ficha].Fila, c, f) then
       begin
         auxf := Nucleo.Juego.Fichas[color, ficha].Fila;
         auxc := Nucleo.Juego.Fichas[color, ficha].Columna;

         {-----------------------------------------------------------------------}
         { Enroque a la derecha. Validamos el movimiento de la torre, para       }
         { saber si hay alguna ficha en la casilla final del rey.                }
         {-----------------------------------------------------------------------}
         if (Nucleo.Juego.Fichas[color, ficha].Columna < c) then
          begin
            if ValidarMovTorre(Nucleo, color, TORRE_DER, (c - 1), f) then
             begin
               {-----------------------------------------------------------------}
               { Movemos el rey una casilla para ver si estara en jaque.        }
               {-----------------------------------------------------------------}
               Nucleo.Juego.Fichas[color, ficha].Columna := (c - 1);
            
               {-----------------------------------------------------------------}
               { Comprobamos si la situacin actual da la posibilidad de que     }
               { se de un jaque mate por accidente al mover el jugador.          }
               {-----------------------------------------------------------------}
               if HayJaque(Nucleo, color, (c - 1), f) then
                begin
                  Nucleo.Juego.Fichas[color, ficha].Columna := auxc;

                  Nucleo.Juego.Fichas[color, TORRE_DER].Columna := 8;
                end
               {-----------------------------------------------------------------}
               { Si todo ha salido bien, ponemos al rey en la posicin final.    }
               {-----------------------------------------------------------------}
               else
                begin
                  Nucleo.Juego.Fichas[color, ficha].Columna := c;

                  {--------------------------------------------------------------}
                  { Volvemos a comprobar si hay jaque en esa posicin.           }
                  {--------------------------------------------------------------}
                  if HayJaque(Nucleo, color, c, f) then
                   begin
                     Nucleo.Juego.Fichas[color, ficha].Columna := auxc;

                     Nucleo.Juego.Fichas[color, TORRE_DER].Columna := 8;
                   end
                  {--------------------------------------------------------------}
                  { Sino todo ha terminado bien.                                 }
                  {--------------------------------------------------------------}
                  else
                   begin
                     resultado := true;
                   end;
                end;
             end;
          end
         {-----------------------------------------------------------------------}
         { Enroque a la izquierda. Validamos el movimiento de la torre, para     }
         { saber si hay alguna ficha en la casilla final del rey.                }
         {-----------------------------------------------------------------------}
         else if (c < Nucleo.Juego.Fichas[color, ficha].Columna) then
          begin
            if ValidarMovTorre(Nucleo, color, TORRE_IZQ, (c + 1), f) then
             begin
               {-----------------------------------------------------------------}
               { Movemos el rey una casilla para ver si estara en jaque.        }
               {-----------------------------------------------------------------}
               Nucleo.Juego.Fichas[color, ficha].Columna := (c + 1);
            
               {-----------------------------------------------------------------}
               { Comprobamos si la situacin actual da la posibilidad de que     }
               { se de un jaque mate por accidente al mover el jugador.          }
               {-----------------------------------------------------------------}
               if HayJaque(Nucleo, color, (c + 1), f) then
                begin
                  Nucleo.Juego.Fichas[color, ficha].Columna := auxc;

                  Nucleo.Juego.Fichas[color, TORRE_DER].Columna := 1;

                  Nucleo.Juego.Mensaje := MSJ_ENROQUE;
                end
               {-----------------------------------------------------------------}
               { Si todo ha salido bien, ponemos al rey en la posicin final.    }
               {-----------------------------------------------------------------}
               else
                begin
                  Nucleo.Juego.Fichas[color, ficha].Columna := c;

                  {--------------------------------------------------------------}
                  { Volvemos a comprobar si hay jaque en esa posicin.           }
                  {--------------------------------------------------------------}
                  if HayJaque(Nucleo, color, c, f) then
                   begin
                     Nucleo.Juego.Fichas[color, ficha].Columna := auxc;

                     Nucleo.Juego.Fichas[color, TORRE_DER].Columna := 1;

                     Nucleo.Juego.Mensaje := MSJ_ENROQUE;
                   end
                  {--------------------------------------------------------------}
                  { Sino todo ha terminado bien.                                 }
                  {--------------------------------------------------------------}
                  else
                   begin
                     resultado := true;
                   end;
                end;
             end;
          end;
       end;
    end
   {-----------------------------------------------------------------------------}
   { Mostramos un mensaje de error.                                              }
   {-----------------------------------------------------------------------------}
   else
    begin
      Nucleo.Juego.Mensaje := MSJ_ENROQUE;
    end;
       
   {-----------------------------------------------------------------------------}
   { Y devolvemos el resultado, rezando para que todo haya ido bien.             }
   {-----------------------------------------------------------------------------}
   ValidarEnroqueRey := resultado;
end;

{********************************************************************************}
{ Funcin que valida el posible movimiento de un rey.                            }
{********************************************************************************}
function ValidarMovRey (var Nucleo: tNucleo; color, ficha, c, f: byte): boolean;
var
   auxc, auxf, tipo: byte;
   fene, cene: byte;
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;

   {-----------------------------------------------------------------------------}
   { Comprobamos que se de alguno de los 8 movimientos simples del rey.          }
   {-----------------------------------------------------------------------------}
   if ((Nucleo.Juego.Fichas[color, ficha].Columna = (c - 1)) and (Nucleo.Juego.Fichas[color, ficha].Fila = (f - 1))) or
      ((Nucleo.Juego.Fichas[color, ficha].Columna = (c - 1)) and (Nucleo.Juego.Fichas[color, ficha].Fila = f))       or
      ((Nucleo.Juego.Fichas[color, ficha].Columna = (c - 1)) and (Nucleo.Juego.Fichas[color, ficha].Fila = (f + 1))) or
      ((Nucleo.Juego.Fichas[color, ficha].Columna = c)       and (Nucleo.Juego.Fichas[color, ficha].Fila = (f - 1))) or
      ((Nucleo.Juego.Fichas[color, ficha].Columna = c)       and (Nucleo.Juego.Fichas[color, ficha].Fila = (f + 1))) or
      ((Nucleo.Juego.Fichas[color, ficha].Columna = (c + 1)) and (Nucleo.Juego.Fichas[color, ficha].Fila = (f - 1))) or
      ((Nucleo.Juego.Fichas[color, ficha].Columna = (c + 1)) and (Nucleo.Juego.Fichas[color, ficha].Fila = f))       or
      ((Nucleo.Juego.Fichas[color, ficha].Columna = (c + 1)) and (Nucleo.Juego.Fichas[color, ficha].Fila = (f + 1))) then
    begin
      ObtenerFicha(Nucleo, c, f, fene, cene);

      auxf := Nucleo.Juego.Fichas[color, ficha].Fila;
      auxc := Nucleo.Juego.Fichas[color, ficha].Columna;

      Nucleo.Juego.Fichas[color, ficha].Fila    := f;
      Nucleo.Juego.Fichas[color, ficha].Columna := c;

      if (color <> cene) and (cene <> 0) then
       begin
         tipo := Nucleo.Juego.Fichas[cene, fene].Tipo;
         Nucleo.Juego.Fichas[cene, fene].Tipo := FICHA_MUERTA;
       end;

      {--------------------------------------------------------------------------}
      { Comprobamos si la situacin actual da la posibilidad de que se de un     }
      { jaque mate por accidente al mover el jugador.                            }
      {--------------------------------------------------------------------------}
      if HayJaque(Nucleo, color, c, f) and (fene <> REY) then
       begin
         Nucleo.Juego.Fichas[color, ficha].Fila    := auxf;
         Nucleo.Juego.Fichas[color, ficha].Columna := auxc;
         
         if (color <> cene) and (cene <> 0) then
            Nucleo.Juego.Fichas[cene, fene].Tipo := tipo;
       end
      {--------------------------------------------------------------------------}
      { Si todo ha salido bien, comprobamos si hemos matado alguien.             }
      {--------------------------------------------------------------------------}
      else
       begin
         {-----------------------------------------------------------------------}
         { Nos encontramos ante una pieza enemiga.                               }
         {-----------------------------------------------------------------------}
         if (color <> cene) and (cene <> 0) then
          begin
            Nucleo.Juego.Fichas[cene, fene].Tipo := FICHA_MUERTA;
            resultado := true;
          end
         {-----------------------------------------------------------------------}
         { Nos encontramos con una casilla vaca.                                }
         {-----------------------------------------------------------------------}
         else if (cene = 0) then
          begin
            resultado := true;
          end;
       end;
    end
   {-----------------------------------------------------------------------------}
   { Comprobamos que est tratando de hacer un enroque.                          }
   {-----------------------------------------------------------------------------}
   else if ((Nucleo.Juego.Fichas[color, ficha].Columna = (c - 2)) and
            (Nucleo.Juego.Fichas[color, ficha].Fila = f) and
            (f = 1) and (color = FICHA_BLANCA) and
            (Nucleo.Juego.Fichas[color, TORRE_DER].Movesp = MOVESP_NINGUNO) and
            (Nucleo.Juego.Fichas[color, ficha].Movesp = MOVESP_NINGUNO)) or

           ((Nucleo.Juego.Fichas[color, ficha].Columna = (c + 2)) and
            (Nucleo.Juego.Fichas[color, ficha].Fila = f) and
            (f = 1) and (color = FICHA_BLANCA) and
            (Nucleo.Juego.Fichas[color, TORRE_IZQ].Movesp = MOVESP_NINGUNO) and
            (Nucleo.Juego.Fichas[color, ficha].Movesp = MOVESP_NINGUNO)) or
            
           ((Nucleo.Juego.Fichas[color, ficha].Columna = (c - 2)) and
            (Nucleo.Juego.Fichas[color, ficha].Fila = f) and
            (f = 8) and (color = FICHA_NEGRA) and
            (Nucleo.Juego.Fichas[color, TORRE_DER].Movesp = MOVESP_NINGUNO) and
            (Nucleo.Juego.Fichas[color, ficha].Movesp = MOVESP_NINGUNO)) or

           ((Nucleo.Juego.Fichas[color, ficha].Columna = (c + 2)) and
            (Nucleo.Juego.Fichas[color, ficha].Fila = f) and
            (f = 8) and (color = FICHA_NEGRA) and
            (Nucleo.Juego.Fichas[color, TORRE_IZQ].Movesp = MOVESP_NINGUNO) and
            (Nucleo.Juego.Fichas[color, ficha].Movesp = MOVESP_NINGUNO)) then
    begin
      resultado := ValidarEnroqueRey(Nucleo, color, ficha, c, f);
    end;

   {-----------------------------------------------------------------------------}
   { Activamos el indicador que imposibilita que se ejecute el enroque.          }
   {-----------------------------------------------------------------------------}
   if (Nucleo.Juego.Fichas[color, ficha].Tipo = FICHA_REY) and resultado then
      Nucleo.Juego.Fichas[color, ficha].Movesp := MOVESP_ENROQUE;

   {-----------------------------------------------------------------------------}
   { Y devolvemos el resultado, rezando para que todo haya ido bien.             }
   {-----------------------------------------------------------------------------}
   ValidarMovRey := resultado;
end;

{********************************************************************************}
{ Funcin que valida el movimiento que se esta tratando de hacer.                }
{********************************************************************************}
function ValidarMovimiento (var Nucleo: tNucleo; color, ficha, c, f: byte): boolean;
var
   ocupada: boolean;
   resultado: boolean;
begin
   {-----------------------------------------------------------------------------}
   { Salvo que ocurra lo contrario presuponemos que va a haber un error.         }
   {-----------------------------------------------------------------------------}
   resultado := false;

   Nucleo.Juego.Mensaje := MSJ_MOVINV;

   {-----------------------------------------------------------------------------}
   { Comprobamos si la casilla destino no esta ocupada por una pieza del mismo   }
   { color de nuestro bando.                                                     }
   {-----------------------------------------------------------------------------}
   ocupada := ValidarCasilla(Nucleo, color, c, f);

   {-----------------------------------------------------------------------------}
   { De no estarlo validamos el movimiento segn el tipo de ficha.               }
   {-----------------------------------------------------------------------------}
   if not ocupada then
      case Nucleo.Juego.Fichas[color, ficha].Tipo of
         FICHA_PEON:    resultado := ValidarMovPeon(Nucleo, color, ficha, c, f);
         FICHA_TORRE:   resultado := ValidarMovTorre(Nucleo, color, ficha, c, f);
         FICHA_CABALLO: resultado := ValidarMovCaballo(Nucleo, color, ficha, c, f);
         FICHA_ALFIL:   resultado := ValidarMovAlfil(Nucleo, color, ficha, c, f);
         FICHA_REINA:   resultado := ValidarMovReina(Nucleo, color, ficha, c, f);
         FICHA_REY:     resultado := ValidarMovRey(Nucleo, color, ficha, c, f);
      end;

   ValidarMovimiento := resultado;
end;

{********************************************************************************}
end.
{********************************************************************************}