{********************************************************************************}
{ Ajedrez 1.0 - Juego del ajedrez bajo ms-dos.                                   }
{ Copyright (C) 2005  Gorka Surez Garca                                        }
{                                                                                }
{ Logica.pas is part of "Ajedrez".                                               }
{                                                                                }
{ "Ajedrez" is free software; you can redistribute it and/or                     }
{ modify it under the terms of the GNU General Public License                    }
{ as published by the Free Software Foundation; either version 2                 }
{ of the License, or (at your option) any later version.                         }
{                                                                                }
{ "Ajedrez" is distributed in the hope that it will be useful,                   }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of                 }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  }
{ GNU General Public License for more details.                                   }
{                                                                                }
{ You should have received a copy of the GNU General Public License              }
{ along with "Ajedrez"; if not, write to the Free Software                       }
{ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.    }
{********************************************************************************}
unit Logica;
{********************************************************************************}
interface {parte publica}
uses Lesslib, Nucleo, Pintado, LogMovs, IAJuego, crt;
{********************************************************************************}
{ Funciones para controlar la lgica del juego.                                  }
{********************************************************************************}
procedure EjecutarLogica (var Nucleo: tNucleo);


{********************************************************************************}
implementation {parte privada}
{********************************************************************************}


{********************************************************************************}
{ Funcin que inicializa los datos de una partida nueva creada.                  }
{********************************************************************************}
procedure CrearPartida (var Nucleo: tNucleo);
var
   i: integer;
begin
   {-----------------------------------------------------------------------------}
   { Inicializamos la casilla seleccionada del tablero.                          }
   {-----------------------------------------------------------------------------}
   Nucleo.Juego.selx := 0;
   Nucleo.Juego.sely := 0;

   {-----------------------------------------------------------------------------}
   { Inicializamos el mensaje para el usuario y el turno.                        }
   {-----------------------------------------------------------------------------}
   Nucleo.Juego.Victoria := VICTORIA_NO;
   Nucleo.Juego.Mensaje  := MSJ_NINGUNO;
   Nucleo.Juego.Turno    := TURNO_BLANCAS;

   Nucleo.Juego.JaqueB := false;
   Nucleo.Juego.JaqueN := false;

   Nucleo.Juego.NoPeon := false;

   {-----------------------------------------------------------------------------}
   { Inicializamos las fichas blancas en el tablero.                             }
   {-----------------------------------------------------------------------------}
   for i := 1 to MAX_PEONES do
    begin
      Nucleo.Juego.Fichas[FICHA_BLANCA, i].Tipo    := FICHA_PEON;
      Nucleo.Juego.Fichas[FICHA_BLANCA, i].Fila    := 2;
      Nucleo.Juego.Fichas[FICHA_BLANCA, i].Columna := i;
      Nucleo.Juego.Fichas[FICHA_BLANCA, i].Movesp  := MOVESP_NINGUNO;
    end;

   for i := (MAX_PEONES + 1) to MAX_FICHAS_JUG do
    begin
      Nucleo.Juego.Fichas[FICHA_BLANCA, i].Fila    := 1;
      Nucleo.Juego.Fichas[FICHA_BLANCA, i].Columna := i - MAX_PEONES;
      Nucleo.Juego.Fichas[FICHA_BLANCA, i].Movesp  := MOVESP_NINGUNO;
    end;

   Nucleo.Juego.Fichas[FICHA_BLANCA, TORRE_IZQ].Tipo   := FICHA_TORRE;
   Nucleo.Juego.Fichas[FICHA_BLANCA, CABALLO_IZQ].Tipo := FICHA_CABALLO;
   Nucleo.Juego.Fichas[FICHA_BLANCA, ALFIL_IZQ].Tipo   := FICHA_ALFIL;
   Nucleo.Juego.Fichas[FICHA_BLANCA, REINA].Tipo       := FICHA_REINA;
   Nucleo.Juego.Fichas[FICHA_BLANCA, REY].Tipo         := FICHA_REY;
   Nucleo.Juego.Fichas[FICHA_BLANCA, ALFIL_DER].Tipo   := FICHA_ALFIL;
   Nucleo.Juego.Fichas[FICHA_BLANCA, CABALLO_DER].Tipo := FICHA_CABALLO;
   Nucleo.Juego.Fichas[FICHA_BLANCA, TORRE_DER].Tipo  := FICHA_TORRE;


   {-----------------------------------------------------------------------------}
   { Inicializamos las fichas negras en el tablero.                              }
   {-----------------------------------------------------------------------------}
   for i := 1 to MAX_PEONES do
    begin
      Nucleo.Juego.Fichas[FICHA_NEGRA, i].Tipo    := FICHA_PEON;
      Nucleo.Juego.Fichas[FICHA_NEGRA, i].Fila    := 7;
      Nucleo.Juego.Fichas[FICHA_NEGRA, i].Columna := i;
      Nucleo.Juego.Fichas[FICHA_NEGRA, i].Movesp  := MOVESP_NINGUNO;
    end;

   for i := (MAX_PEONES + 1) to MAX_FICHAS_JUG do
    begin
      Nucleo.Juego.Fichas[FICHA_NEGRA, i].Fila    := 8;
      Nucleo.Juego.Fichas[FICHA_NEGRA, i].Columna := i - MAX_PEONES;
      Nucleo.Juego.Fichas[FICHA_NEGRA, i].Movesp  := MOVESP_NINGUNO;
    end;

   Nucleo.Juego.Fichas[FICHA_NEGRA, TORRE_IZQ].Tipo   := FICHA_TORRE;
   Nucleo.Juego.Fichas[FICHA_NEGRA, CABALLO_IZQ].Tipo := FICHA_CABALLO;
   Nucleo.Juego.Fichas[FICHA_NEGRA, ALFIL_IZQ].Tipo   := FICHA_ALFIL;
   Nucleo.Juego.Fichas[FICHA_NEGRA, REINA].Tipo       := FICHA_REINA;
   Nucleo.Juego.Fichas[FICHA_NEGRA, REY].Tipo         := FICHA_REY;
   Nucleo.Juego.Fichas[FICHA_NEGRA, ALFIL_DER].Tipo   := FICHA_ALFIL;
   Nucleo.Juego.Fichas[FICHA_NEGRA, CABALLO_DER].Tipo := FICHA_CABALLO;
   Nucleo.Juego.Fichas[FICHA_NEGRA, TORRE_DER].Tipo   := FICHA_TORRE;

   
   {-----------------------------------------------------------------------------}
   { Inicializamos otras variables importantes para el juego.                    }
   {-----------------------------------------------------------------------------}
   Nucleo.Juego.EstadoIA := ESTADOIA_NORMAL;
end;

{********************************************************************************}
{ Funcin que crea una partida para un jugador.                                  }
{********************************************************************************}
procedure CrearPartida1Jug (var Nucleo: tNucleo);
begin
   Nucleo.Juego.Estado := EST_1JUG;

   CrearPartida(Nucleo);
end;

{********************************************************************************}
{ Funcin que crea una partida para dos jugadores.                               }
{********************************************************************************}
procedure CrearPartida2Jug (var Nucleo: tNucleo);
begin
   Nucleo.Juego.Estado := EST_2JUG;

   CrearPartida(Nucleo);
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla del men.                                     }
{********************************************************************************}
procedure LogicaMenu (var Nucleo: tNucleo);
var
 tecla: integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "1 Jugador" ha sido pulsado.                     }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.MnuBotones[BOT_MNU_1JUGADORES]) then
         CrearPartida1Jug(Nucleo);

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "2 Jugadores" ha sido pulsado.                   }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.MnuBotones[BOT_MNU_2JUGADORES]) then
         CrearPartida2Jug(Nucleo);

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "Cargar" ha sido pulsado.                        }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.MnuBotones[BOT_MNU_CARGAR]) then
         Nucleo.Juego.Estado := EST_CARGAR1;

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "Ayuda" ha sido pulsado.                         }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.MnuBotones[BOT_MNU_AYUDA]) then
         Nucleo.Juego.Estado := EST_AYUDA1;

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "Salir" ha sido pulsado.                         }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.MnuBotones[BOT_MNU_SALIR]) then
         Nucleo.Juego.Estado := EST_SALIR1;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Si es ESC vamos a la pantalla de salir del juego.                        }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_SALIR1;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona que botn del tablero ha sido pulsado.                    }
{********************************************************************************}
function DarBotonTablero (var Nucleo: tNucleo): integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn "Reiniciar" ha sido pulsado.                        }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.TabBotones[BOT_TAB_REINICIAR]) then
      DarBotonTablero := BOT_REINICIAR;

   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn "Salvar" ha sido pulsado.                           }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.TabBotones[BOT_TAB_SALVAR]) then
      DarBotonTablero := BOT_SALVAR;

   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn "Cargar" ha sido pulsado.                           }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.TabBotones[BOT_TAB_CARGAR]) then
      DarBotonTablero := BOT_CARGAR;

   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn "Ayuda" ha sido pulsado.                            }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.TabBotones[BOT_TAB_AYUDA]) then
      DarBotonTablero := BOT_AYUDA;

   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn "Salir" ha sido pulsado.                            }
   {-----------------------------------------------------------------------------}
   if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.TabBotones[BOT_TAB_SALIR]) then
      DarBotonTablero := BOT_SALIR;
end;

{********************************************************************************}
{ Funcin que gestiona que casilla del tablero ha sido pulsada.                  }
{********************************************************************************}
procedure DarCasilla (var Nucleo: tNucleo; var x, y: byte);
var
   i, j: byte;
   reg: tRect;
begin
   x := 0;
   y := 0;

   {-----------------------------------------------------------------------------}
   { Comprobamos si el ratn est encima de cada una de las casillas.            }
   {-----------------------------------------------------------------------------}
   for i := 1 to MAX_FILAS do
      for j := 1 to MAX_COLUMNAS do
       begin
         reg.x := (j - 1) * TAB_SEP_X + TAB_POS_X;
         reg.y := (8 - i) * TAB_SEP_Y + TAB_POS_Y;
         reg.w := TAB_SEP_X;
         reg.h := TAB_SEP_Y;

         if RatonEstaEn2(Nucleo.Raton, reg) then
          begin
            x := j;
            y := i;
          end;
       end;
end;

{********************************************************************************}
{ Funcin que gestiona una partida de un solo jugador.                           }
{********************************************************************************}
procedure Logica1Jug (var Nucleo: tNucleo);
var
   tecla: integer;
   c, f, ficha: byte;
begin
   {-----------------------------------------------------------------------------}
   { Echamos un vistazo al turno actual.                                         }
   {-----------------------------------------------------------------------------}
   if Nucleo.Juego.Turno = TURNO_BLANCAS then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn izquierdo del ratn est pulsado.                }
      {--------------------------------------------------------------------------}
      if Nucleo.Raton.bi then
       begin
         {-----------------------------------------------------------------------}
         { Comprobamos primero si ha pulsado alguno de los botones.              }
         {-----------------------------------------------------------------------}
         case DarBotonTablero(Nucleo) of
            BOT_REINICIAR: Nucleo.Juego.Estado := EST_REINICIAR1;
            BOT_SALVAR:    Nucleo.Juego.Estado := EST_SALVAR1;
            BOT_CARGAR:    Nucleo.Juego.Estado := EST_CARGAR2;
            BOT_AYUDA:     Nucleo.Juego.Estado := EST_AYUDA2;
            BOT_SALIR:     Nucleo.Juego.Estado := EST_SALIR2;
         end;

         {-----------------------------------------------------------------------}
         { Obtenemos la casilla que hay pulsada actualmente.                     }
         {-----------------------------------------------------------------------}
         DarCasilla(Nucleo, c, f);

         {-----------------------------------------------------------------------}
         { Y si haba ya una casilla pulsada con anterioridad...                 }
         {-----------------------------------------------------------------------}
         if (0 < Nucleo.Juego.selx) and (Nucleo.Juego.selx < 9) and
            (0 < Nucleo.Juego.sely) and (Nucleo.Juego.sely < 9) then
          begin
            {--------------------------------------------------------------------}
            { Si la casilla pulsada es la misma que la anterior, la borramos.    }
            {--------------------------------------------------------------------}
            if (Nucleo.Juego.selx = c) and (Nucleo.Juego.sely = f) then
             begin
               Nucleo.Juego.selx := 0;
               Nucleo.Juego.sely := 0;

               Nucleo.Juego.Mensaje := MSJ_NINGUNO;
             end
            {--------------------------------------------------------------------}
            { Si no lo es, obtenemos la ficha y validamos su movimiento.         }
            {--------------------------------------------------------------------}
            else
             begin
               ficha := DarNumFicha(Nucleo, Nucleo.Juego.Turno, Nucleo.Juego.selx,
                                    Nucleo.Juego.sely);

               if ValidarMovimiento(Nucleo, Nucleo.Juego.Turno, ficha, c, f) then
                begin
                  Nucleo.Juego.selx := 0;
                  Nucleo.Juego.sely := 0;

                  Nucleo.Juego.Mensaje := MSJ_NINGUNO;

                  CambiarTurno(Nucleo);
                end;
             end;
          end
         {-----------------------------------------------------------------------}
         { Si no haba ninguna casilla pulsada comprobamos que corresponde a una }
         { ficha del jugador.                                                    }
         {-----------------------------------------------------------------------}
         else
          begin
            if ValidarCasilla(Nucleo, Nucleo.Juego.Turno, c, f) then
             begin
               Nucleo.Juego.selx := c;
               Nucleo.Juego.sely := f;

               Nucleo.Juego.Mensaje := MSJ_NINGUNO;
             end;
          end;

         {-----------------------------------------------------------------------}
         { Retardo para evitar saltos indeseados.                                }
         {-----------------------------------------------------------------------}
         DoDelay(TIEMPO);
       end
      else
       begin
         {-----------------------------------------------------------------------}
         { Comprobamos el valor de la ltima tecla pulsada.                      }
         {-----------------------------------------------------------------------}
         tecla := DarTecla;

         {-----------------------------------------------------------------------}
         { Si es ESC vamos a la pantalla de salir de la partida.                 }
         {-----------------------------------------------------------------------}
         if tecla = TECLA_ESC then
            Nucleo.Juego.Estado := EST_SALIR2;
       end;
    end
   {-----------------------------------------------------------------------------}
   { Si es el de las negras mueve la mquina.                                    }
   {-----------------------------------------------------------------------------}
   else
    begin
      {--------------------------------------------------------------------------}
      { Pero movemos siempre y cuando no estemos en jaque mate.                  }
      {--------------------------------------------------------------------------}
      if not Nucleo.Juego.JaqueN then begin
         MaquinaMueve(Nucleo);
         CambiarTurno(Nucleo);

         if Nucleo.Juego.JaqueB then begin
            PintarPantalla(Nucleo);
            DoDelay(TIEMPO * 2);
            Nucleo.Juego.Victoria := VICTORIA_J2;
         end;

      {--------------------------------------------------------------------------}
      { Si lo estamos gana el jugador y pierde la mquina.                       }
      {--------------------------------------------------------------------------}
      end else begin
         Nucleo.Juego.Victoria := VICTORIA_J1;
      end;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona el cuadro de dialogo de victoria en un solo jugador.      }
{********************************************************************************}
procedure DlgVictoria1Jug (var Nucleo: tNucleo);
var
   tecla: integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "S" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
         CrearPartida1Jug(Nucleo);

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "No" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
         Nucleo.Juego.Estado := EST_MENU;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Comprobamos si la tecla "ESC" ha sido pulsada.                           }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_MENU;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona una partida de un solo jugador.                           }
{********************************************************************************}
procedure Partida1Jug (var Nucleo: tNucleo);
begin
   case Nucleo.Juego.Victoria of
      VICTORIA_NO:  Logica1Jug(Nucleo);
      VICTORIA_J1:  DlgVictoria1Jug(Nucleo);
      VICTORIA_J2:  DlgVictoria1Jug(Nucleo);
      JUEGO_TABLAS: DlgVictoria1Jug(Nucleo);
   end;
end;

{********************************************************************************}
{ Funcin que gestiona una partida de dos jugadores.                             }
{********************************************************************************}
procedure Logica2Jug (var Nucleo: tNucleo);
var
   tecla: integer;
   c, f, ficha: byte;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos primero si ha pulsado alguno de los botones.                 }
      {--------------------------------------------------------------------------}
      case DarBotonTablero(Nucleo) of
         BOT_REINICIAR: Nucleo.Juego.Estado := EST_REINICIAR2;
         BOT_SALVAR:    Nucleo.Juego.Estado := EST_SALVAR2;
         BOT_CARGAR:    Nucleo.Juego.Estado := EST_CARGAR3;
         BOT_AYUDA:     Nucleo.Juego.Estado := EST_AYUDA3;
         BOT_SALIR:     Nucleo.Juego.Estado := EST_SALIR3;
      end;

      {--------------------------------------------------------------------------}
      { Obtenemos la casilla que hay pulsada actualmente.                        }
      {--------------------------------------------------------------------------}
      DarCasilla(Nucleo, c, f);

      {--------------------------------------------------------------------------}
      { Y si haba ya una casilla pulsada con anterioridad...                    }
      {--------------------------------------------------------------------------}
      if (0 < Nucleo.Juego.selx) and (Nucleo.Juego.selx < 9) and
         (0 < Nucleo.Juego.sely) and (Nucleo.Juego.sely < 9) then
       begin
         {-----------------------------------------------------------------------}
         { Si la casilla pulsada es la misma que la anterior, la borramos.       }
         {-----------------------------------------------------------------------}
         if (Nucleo.Juego.selx = c) and (Nucleo.Juego.sely = f) then
          begin
            Nucleo.Juego.selx := 0;
            Nucleo.Juego.sely := 0;

            Nucleo.Juego.Mensaje := MSJ_NINGUNO;
          end
         {-----------------------------------------------------------------------}
         { Si no lo es, obtenemos la ficha y validamos su movimiento.            }
         {-----------------------------------------------------------------------}
         else
          begin
            ficha := DarNumFicha(Nucleo, Nucleo.Juego.Turno, Nucleo.Juego.selx,
                                 Nucleo.Juego.sely);

            if ValidarMovimiento(Nucleo, Nucleo.Juego.Turno, ficha, c, f) then
             begin
               Nucleo.Juego.selx := 0;
               Nucleo.Juego.sely := 0;

               Nucleo.Juego.Mensaje := MSJ_NINGUNO;

               CambiarTurno(Nucleo);
             end;
          end;
       end
      {--------------------------------------------------------------------------}
      { Si no haba ninguna casilla pulsada comprobamos que corresponde a una    }
      { ficha del jugador.                                                       }
      {--------------------------------------------------------------------------}
      else
       begin
         if ValidarCasilla(Nucleo, Nucleo.Juego.Turno, c, f) then
          begin
            Nucleo.Juego.selx := c;
            Nucleo.Juego.sely := f;

            Nucleo.Juego.Mensaje := MSJ_NINGUNO;
          end;
       end;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Si es ESC vamos a la pantalla de salir de la partida.                    }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_SALIR3;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona el cuadro de dialogo de victoria en un solo jugador.      }
{********************************************************************************}
procedure DlgVictoria2Jug (var Nucleo: tNucleo);
var
   tecla: integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "S" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
         CrearPartida2Jug(Nucleo);

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "No" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
         Nucleo.Juego.Estado := EST_MENU;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Comprobamos si la tecla "ESC" ha sido pulsada.                           }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_MENU;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona una partida de un solo jugador.                           }
{********************************************************************************}
procedure Partida2Jug (var Nucleo: tNucleo);
begin
   case Nucleo.Juego.Victoria of
      VICTORIA_NO:  Logica2Jug(Nucleo);
      VICTORIA_J1:  DlgVictoria2Jug(Nucleo);
      VICTORIA_J2:  DlgVictoria2Jug(Nucleo);
      JUEGO_TABLAS: DlgVictoria2Jug(Nucleo);
   end;
end;

{********************************************************************************}
{ Funcin que da el nmero de los nombres de las partidas guardadas pinchado.    }
{********************************************************************************}
procedure DarPartidaSeleccionada (var Nucleo: tNucleo);
var
   i: byte;
   r: tRect;
begin
   Nucleo.Partidas.Parsel := 0;

   {-----------------------------------------------------------------------------}
   { Vamos uno por uno comprobando cual ha sido pinchado.                        }
   {-----------------------------------------------------------------------------}
   for i := 1 to MAX_PARTIDAS do
    begin
      r.x := PART_INITX;
      r.y := PART_INITY + PART_SEP_Y * (i - 1);
      r.w := PART_INITW;
      r.h := PART_INITH;

      if RatonEstaEn2(Nucleo.Raton, r) then
         Nucleo.Partidas.Parsel := i;
    end;
end;

{********************************************************************************}
{ Funcin que salva una partida.                                                 }
{********************************************************************************}
procedure SalvarPartida (var Nucleo: tNucleo);
var
   f: tFichPartida;
   reg: tRegPartida;
begin
   {-----------------------------------------------------------------------------}
   { Asignamos el fichero a la variable f.                                       }
   {-----------------------------------------------------------------------------}
   assign(f, Nucleo.Partidas.Fichero[Nucleo.Partidas.Parsel]);

   {-----------------------------------------------------------------------------}
   { Lo abrimos para escribir.                                                   }
   {-----------------------------------------------------------------------------}
   {$I-}
   rewrite(f);
   {$I+}
   
   {-----------------------------------------------------------------------------}
   { Si no hay ningn fallo escribimos su contenido.                             }
   {-----------------------------------------------------------------------------}
   if IOResult = 0 then
    begin
      reg.Nombre := Nucleo.Partidas.Nombre[Nucleo.Partidas.Parsel];
      reg.Juego := Nucleo.Juego;

      if Nucleo.Juego.Estado = EST_SALVAR1 then
         reg.Juego.Estado := EST_1JUG
      else if Nucleo.Juego.Estado = EST_SALVAR2 then
         reg.Juego.Estado := EST_2JUG;

      write(f, reg);      

      close(f);
    end;
end;

{********************************************************************************}
{ Funcin que carga una partida.                                                 }
{********************************************************************************}
procedure CargarPartida (var Nucleo: tNucleo);
var
   f: tFichPartida;
   reg: tRegPartida;
begin
   {-----------------------------------------------------------------------------}
   { Asignamos el fichero a la variable f.                                       }
   {-----------------------------------------------------------------------------}
   assign(f, Nucleo.Partidas.Fichero[Nucleo.Partidas.Parsel]);

   {-----------------------------------------------------------------------------}
   { Lo abrimos para leer.                                                       }
   {-----------------------------------------------------------------------------}
   {$I-}
   reset(f);
   {$I+}
   
   {-----------------------------------------------------------------------------}
   { Si no hay ningn fallo leemos su contenido.                                 }
   {-----------------------------------------------------------------------------}
   if IOResult = 0 then
    begin
      if filesize(f) > 0 then
       begin
         read(f, reg);
         Nucleo.Partidas.Nombre[Nucleo.Partidas.Parsel] := reg.Nombre;
         reg.Juego.Estado := Nucleo.Juego.Estado;
         Nucleo.Juego := reg.Juego;
       end;

      close(f);
    end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de salvar una partida.                        }
{********************************************************************************}
procedure LogicaSalvar (var Nucleo: tNucleo);
var
   tecla, tam, i: integer;
   auxcad: string;
begin
   case Nucleo.Partidas.Estado of
      {--------------------------------------------------------------------------}
      { Pantalla normal de presentacin de salvar partidas.                      }
      {--------------------------------------------------------------------------}
      PART_EST_NORMAL:
       begin
         if Nucleo.Raton.bi then
          begin
            {--------------------------------------------------------------------}
            { Buscamos la partida seleccionada.                                  }
            {--------------------------------------------------------------------}
            DarPartidaSeleccionada(Nucleo);

            if Nucleo.Partidas.Parsel <> 0 then
             begin
               if Nucleo.Partidas.Nombre[Nucleo.Partidas.Parsel] = '---' then
                begin
                  Nucleo.Partidas.Auxcad := '';
                  Nucleo.Partidas.Estado := PART_EST_METER
                end
               else
                begin
                  Nucleo.Partidas.Auxcad := Nucleo.Partidas.Nombre[Nucleo.Partidas.Parsel];
                  Nucleo.Partidas.Estado := PART_EST_AVISO;
                end;
             end;

            {--------------------------------------------------------------------}
            { Retardo para evitar saltos indeseados.                             }
            {--------------------------------------------------------------------}
            DoDelay(TIEMPO);
          end
         else
          begin
            tecla := DarTecla;

            {--------------------------------------------------------------------}
            { Si pulsa escape salimos de la pantalla.                            }
            {--------------------------------------------------------------------}
            if tecla = TECLA_ESC then
             begin
               Nucleo.Partidas.Estado := PART_EST_NORMAL;

               if Nucleo.Juego.Estado = EST_SALVAR1 then
                  Nucleo.Juego.Estado := EST_1JUG
               else if Nucleo.Juego.Estado = EST_SALVAR2 then
                  Nucleo.Juego.Estado := EST_2JUG;
             end;
          end;
       end;

      {--------------------------------------------------------------------------}
      { Aviso sobre si desea sobre-escribir una partida.                         }
      {--------------------------------------------------------------------------}
      PART_EST_AVISO:
       begin
         if Nucleo.Raton.bi then
          begin
            {--------------------------------------------------------------------}
            { Comprobamos si el botn "S" ha sido pulsado.                      }
            {--------------------------------------------------------------------}
            if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
               Nucleo.Partidas.Estado := PART_EST_METER;

            {--------------------------------------------------------------------}
            { Comprobamos si el botn "No" ha sido pulsado.                      }
            {--------------------------------------------------------------------}
            if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
               Nucleo.Partidas.Estado := PART_EST_NORMAL;

            {--------------------------------------------------------------------}
            { Retardo para evitar saltos indeseados.                             }
            {--------------------------------------------------------------------}
            DoDelay(TIEMPO);
          end
         else
          begin
            tecla := DarTecla;

            {--------------------------------------------------------------------}
            { Si pulsa escape salimos del dilogo.                               }
            {--------------------------------------------------------------------}
            if tecla = TECLA_ESC then
             begin
               Nucleo.Partidas.Estado := PART_EST_NORMAL;
             end;
          end;
       end;

      {--------------------------------------------------------------------------}
      { Seccin para meter el nombre de la partida.                              }
      {--------------------------------------------------------------------------}
      PART_EST_METER:
       begin
         tecla := DarTecla;
         
         {-----------------------------------------------------------------------}
         { Si pulsa escape salimos de la pantalla.                               }
         {-----------------------------------------------------------------------}
         if tecla = TECLA_ESC then
          begin
            Nucleo.Partidas.Estado := PART_EST_NORMAL;

            if Nucleo.Juego.Estado = EST_SALVAR1 then
               Nucleo.Juego.Estado := EST_1JUG
            else if Nucleo.Juego.Estado = EST_SALVAR2 then
               Nucleo.Juego.Estado := EST_2JUG
          end
         {-----------------------------------------------------------------------}
         { Si pulsa enter salvamos la partida.                                   }
         {-----------------------------------------------------------------------}
         else if (tecla = TECLA_ENTER) and (Nucleo.Partidas.Auxcad <> '') then
          begin
            Nucleo.Partidas.Nombre[Nucleo.Partidas.Parsel] := Nucleo.Partidas.Auxcad;

            SalvarPartida(Nucleo);

            Nucleo.Partidas.Estado := PART_EST_NORMAL;

            if Nucleo.Juego.Estado = EST_SALVAR1 then
               Nucleo.Juego.Estado := EST_1JUG
            else if Nucleo.Juego.Estado = EST_SALVAR2 then
               Nucleo.Juego.Estado := EST_2JUG;
          end
         {-----------------------------------------------------------------------}
         { Si pulsamos retroceso borramos un caracter.                           }
         {-----------------------------------------------------------------------}
         else if (tecla = TECLA_RETROCESO) and (Nucleo.Partidas.Auxcad <> '') then
          begin
            tam := length(Nucleo.Partidas.Auxcad) - 1;
            auxcad := Nucleo.Partidas.Auxcad;
            Nucleo.Partidas.Auxcad := '';

            for i := 1 to tam do
               Nucleo.Partidas.Auxcad := Nucleo.Partidas.Auxcad + auxcad[i]; 
          end
         {-----------------------------------------------------------------------}
         { Y sino metemos el caracter que sea en la cadena.                      }
         {-----------------------------------------------------------------------}
         else if (tecla <> 0) and (tecla <> TECLA_ENTER) and (tecla <> TECLA_RETROCESO) then
          begin
            Nucleo.Partidas.Auxcad := Nucleo.Partidas.Auxcad + chr(tecla);
            DoDelay(TIEMPO);
          end;
       end;
   end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de cargar una partida contra la maquina.      }
{********************************************************************************}
procedure LogicaCargar (var Nucleo: tNucleo);
var
   tecla: integer;
begin
   case Nucleo.Partidas.Estado of
      {--------------------------------------------------------------------------}
      { Pantalla normal de presentacin de salvar partidas.                      }
      {--------------------------------------------------------------------------}
      PART_EST_NORMAL:
       begin
         if Nucleo.Raton.bi then
          begin
            {--------------------------------------------------------------------}
            { Buscamos la partida seleccionada.                                  }
            {--------------------------------------------------------------------}
            DarPartidaSeleccionada(Nucleo);

            if Nucleo.Partidas.Parsel <> 0 then
               if Nucleo.Partidas.Nombre[Nucleo.Partidas.Parsel] <> '---' then
                  Nucleo.Partidas.Estado := PART_EST_AVISO;

            {--------------------------------------------------------------------}
            { Retardo para evitar saltos indeseados.                             }
            {--------------------------------------------------------------------}
            DoDelay(TIEMPO);
          end
         else
          begin
            tecla := DarTecla;

            {--------------------------------------------------------------------}
            { Si pulsa escape salimos de la pantalla.                            }
            {--------------------------------------------------------------------}
            if tecla = TECLA_ESC then
             begin
               Nucleo.Partidas.Estado := PART_EST_NORMAL;

               if Nucleo.Juego.Estado = EST_CARGAR1 then
                  Nucleo.Juego.Estado := EST_MENU
               else if Nucleo.Juego.Estado = EST_CARGAR2 then
                  Nucleo.Juego.Estado := EST_1JUG
               else if Nucleo.Juego.Estado = EST_CARGAR3 then
                  Nucleo.Juego.Estado := EST_2JUG;
             end;
          end;
       end;

      {--------------------------------------------------------------------------}
      { Aviso sobre si desea cargar una partida.                                 }
      {--------------------------------------------------------------------------}
      PART_EST_AVISO:
       begin
         if Nucleo.Raton.bi then
          begin
            {--------------------------------------------------------------------}
            { Comprobamos si el botn "S" ha sido pulsado.                      }
            {--------------------------------------------------------------------}
            if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
             begin
               Nucleo.Partidas.Estado := PART_EST_NORMAL;

               CargarPartida(Nucleo);
               
               if Nucleo.Juego.Estado = EST_CARGAR1 then
                  Nucleo.Juego.Estado := EST_1JUG
               else if Nucleo.Juego.Estado = EST_CARGAR2 then
                  Nucleo.Juego.Estado := EST_1JUG
               else if Nucleo.Juego.Estado = EST_CARGAR3 then
                  Nucleo.Juego.Estado := EST_2JUG;
             end;

            {--------------------------------------------------------------------}
            { Comprobamos si el botn "No" ha sido pulsado.                      }
            {--------------------------------------------------------------------}
            if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
               Nucleo.Partidas.Estado := PART_EST_NORMAL;

            {--------------------------------------------------------------------}
            { Retardo para evitar saltos indeseados.                             }
            {--------------------------------------------------------------------}
            DoDelay(TIEMPO);
          end
         else
          begin
            tecla := DarTecla;

            {--------------------------------------------------------------------}
            { Si pulsa escape salimos del dilogo.                               }
            {--------------------------------------------------------------------}
            if tecla = TECLA_ESC then
             begin
               Nucleo.Partidas.Estado := PART_EST_NORMAL;
             end;
          end;
       end;
   end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de ayuda.                                     }
{********************************************************************************}
procedure LogicaAyuda1 (var Nucleo: tNucleo);
begin
   Nucleo.Juego.Estado := EST_MENU;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de ayuda en una partida contra la maquina.    }
{********************************************************************************}
procedure LogicaAyuda2 (var Nucleo: tNucleo);
begin
   Nucleo.Juego.Estado := EST_1JUG;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de ayuda en una partida contra otro jugador.  }
{********************************************************************************}
procedure LogicaAyuda3 (var Nucleo: tNucleo);
begin
   Nucleo.Juego.Estado := EST_2JUG;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de salir del juego.                           }
{********************************************************************************}
procedure LogicaSalir1 (var Nucleo: tNucleo);
var
   tecla: integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "S" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
         Nucleo.Juego.Estado := EST_FIN;

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "No" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
         Nucleo.Juego.Estado := EST_MENU;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Comprobamos si la tecla "ESC" ha sido pulsada.                           }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_MENU;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de salir de la partida contra la maquina.     }
{********************************************************************************}
procedure LogicaSalir2 (var Nucleo: tNucleo);
var
   tecla: integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "S" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
         Nucleo.Juego.Estado := EST_MENU;

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "No" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
         Nucleo.Juego.Estado := EST_1JUG;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Comprobamos si la tecla "ESC" ha sido pulsada.                           }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_1JUG;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de salir de la partida contra otro jugador.   }
{********************************************************************************}
procedure LogicaSalir3 (var Nucleo: tNucleo);
var
   tecla: integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "S" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
         Nucleo.Juego.Estado := EST_MENU;

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "No" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
         Nucleo.Juego.Estado := EST_2JUG;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Comprobamos si la tecla "ESC" ha sido pulsada.                           }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_2JUG;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de reiniciar la partida contra la maquina.    }
{********************************************************************************}
procedure LogicaReiniciar1 (var Nucleo: tNucleo);
var
   tecla: integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "S" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
         CrearPartida1Jug(Nucleo);

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "No" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
         Nucleo.Juego.Estado := EST_1JUG;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Comprobamos si la tecla "ESC" ha sido pulsada.                           }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_1JUG;
    end;
end;

{********************************************************************************}
{ Funcin que gestiona la pantalla de reiniciar la partida contra otro jugador.  }
{********************************************************************************}
procedure LogicaReiniciar2 (var Nucleo: tNucleo);
var
   tecla: integer;
begin
   {-----------------------------------------------------------------------------}
   { Comprobamos si el botn izquierdo del ratn est pulsado.                   }
   {-----------------------------------------------------------------------------}
   if Nucleo.Raton.bi then
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "S" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_SI]) then
         CrearPartida2Jug(Nucleo);

      {--------------------------------------------------------------------------}
      { Comprobamos si el botn "No" ha sido pulsado.                            }
      {--------------------------------------------------------------------------}
      if RatonEstaEn2(Nucleo.Raton, Nucleo.Texturas.DlgBotones[BOT_DLG_NO]) then
         Nucleo.Juego.Estado := EST_2JUG;

      {--------------------------------------------------------------------------}
      { Retardo para evitar saltos indeseados.                                   }
      {--------------------------------------------------------------------------}
      DoDelay(TIEMPO);
    end
   else
    begin
      {--------------------------------------------------------------------------}
      { Comprobamos el valor de la ltima tecla pulsada.                         }
      {--------------------------------------------------------------------------}
      tecla := DarTecla;

      {--------------------------------------------------------------------------}
      { Comprobamos si la tecla "ESC" ha sido pulsada.                           }
      {--------------------------------------------------------------------------}
      if tecla = TECLA_ESC then
         Nucleo.Juego.Estado := EST_2JUG;
    end;
end;

{********************************************************************************}
{ Funcin principal que gestiona toda la lgica interna del juego.               }
{********************************************************************************}
procedure EjecutarLogica (var Nucleo: tNucleo);
begin
   RatonEstado(Nucleo.Raton);

   case Nucleo.Juego.Estado of
      EST_MENU:       LogicaMenu(Nucleo);
      EST_1JUG:       Partida1Jug(Nucleo);
      EST_2JUG:       Partida2Jug(Nucleo);
      EST_SALVAR1:    LogicaSalvar(Nucleo);
      EST_SALVAR2:    LogicaSalvar(Nucleo);
      EST_CARGAR1:    LogicaCargar(Nucleo);
      EST_CARGAR2:    LogicaCargar(Nucleo);
      EST_CARGAR3:    LogicaCargar(Nucleo);
      EST_AYUDA1:     LogicaAyuda1(Nucleo);
      EST_AYUDA2:     LogicaAyuda2(Nucleo);
      EST_AYUDA3:     LogicaAyuda3(Nucleo);
      EST_SALIR1:     LogicaSalir1(Nucleo);
      EST_SALIR2:     LogicaSalir2(Nucleo);
      EST_SALIR3:     LogicaSalir3(Nucleo);
      EST_REINICIAR1: LogicaReiniciar1(Nucleo);
      EST_REINICIAR2: LogicaReiniciar2(Nucleo);
   end;
end;

{********************************************************************************}
end.
{********************************************************************************}