{********************************************************************************}
{ Ajedrez 1.0 - Juego del ajedrez bajo ms-dos.                                   }
{ Copyright (C) 2005  Gorka Surez Garca                                        }
{                                                                                }
{ Nucleo.pas is part of "Ajedrez".                                               }
{                                                                                }
{ "Ajedrez" is free software; you can redistribute it and/or                     }
{ modify it under the terms of the GNU General Public License                    }
{ as published by the Free Software Foundation; either version 2                 }
{ of the License, or (at your option) any later version.                         }
{                                                                                }
{ "Ajedrez" is distributed in the hope that it will be useful,                   }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of                 }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  }
{ GNU General Public License for more details.                                   }
{                                                                                }
{ You should have received a copy of the GNU General Public License              }
{ along with "Ajedrez"; if not, write to the Free Software                       }
{ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.    }
{********************************************************************************}
unit Nucleo;
{********************************************************************************}
interface {parte publica}
uses Lesslib, crt;
{********************************************************************************}
{ Constantes del juego.                                                          }
{********************************************************************************}
const
   {-----------------------------------------------------------------------------}
   { Constantes relacionadas con las fichas del ajedrez.                         }
   {-----------------------------------------------------------------------------}
   FICHA_BLANCA = 1;
   FICHA_NEGRA  = 2;

   FICHA_MUERTA  = 0;
   FICHA_PEON    = 1;
   FICHA_TORRE   = 2;
   FICHA_CABALLO = 3;
   FICHA_ALFIL   = 4;
   FICHA_REINA   = 5;
   FICHA_REY     = 6;

   MAX_FICHAS     = 6;
   MAX_FICHAS_JUG = 16;
   MAX_PEONES     = 8;

   FICHA_ANCHO = 20;
   FICHA_ALTO  = 20;

   MOVESP_NINGUNO = 0;
   MOVESP_PEON_A  = 1; {Ataque especial}
   MOVESP_PEON_V  = 2; {Victima del ataque}
   MOVESP_ENROQUE = 3;

   TORRE_IZQ   =  9;
   CABALLO_IZQ = 10;
   ALFIL_IZQ   = 11;
   REINA       = 12;
   REY         = 13;
   ALFIL_DER   = 14;
   CABALLO_DER = 15;
   TORRE_DER   = 16;

   {-----------------------------------------------------------------------------}
   { Constantes relacionadas con los turnos del juego.                           }
   {-----------------------------------------------------------------------------}
   TURNO_BLANCAS = 1;
   TURNO_NEGRAS  = 2;

   {-----------------------------------------------------------------------------}
   { Constantes relacionadas con el tablero del ajedrez.                         }
   {-----------------------------------------------------------------------------}
   MAX_FILAS    = 8;
   MAX_COLUMNAS = 8;

   TAB_POS_X = 20;
   TAB_POS_Y = 20;

   TAB_SEP_X = 20;
   TAB_SEP_Y = 20;

   {-----------------------------------------------------------------------------}
   { Constantes de la IA del juego.                                              }
   {-----------------------------------------------------------------------------}
   {MAX_MOVIMIENTOS  = 226;}
   
   ESTADOIA_APERTURA = 0;
   ESTADOIA_NORMAL   = 1;
   ESTADOIA_FINREINA = 2;
   ESTADOIA_FINTORRE = 3;
   ESTADOIA_FINALFIL = 4;

   MAX_MOVS_PEON    =  4;
   MAX_MOVS_TORRE   = 28;
   MAX_MOVS_CABALLO =  8;
   MAX_MOVS_ALFIL   = 28;
   MAX_MOVS_REINA   = 56;
   MAX_MOVS_REY     = 10;

   {-----------------------------------------------------------------------------}
   { Constantes relacionadas con los botones de la interfaz.                     }
   {-----------------------------------------------------------------------------}
   MAX_BOTONES = 11;
   MNU_BOTONES =  5;
   TAB_BOTONES =  5;
   DLG_BOTONES =  2;

   BOT_NINGUNO    =  0;
   BOT_1JUGADORES =  1;
   BOT_2JUGADORES =  2;
   BOT_REINICIAR  =  3;
   BOT_SALVAR     =  4;
   BOT_CARGAR     =  5;
   BOT_AYUDA      =  6;
   BOT_SALIR      =  7;
   BOT_SI_NORMAL  =  8;
   BOT_SI         =  9;
   BOT_NO_NORMAL  = 10;
   BOT_NO         = 11;

   BOT_MNU_1JUGADORES = 1;
   BOT_MNU_2JUGADORES = 2;
   BOT_MNU_CARGAR     = 3;
   BOT_MNU_AYUDA      = 4;
   BOT_MNU_SALIR      = 5;

   BOT_TAB_REINICIAR  = 1;
   BOT_TAB_SALVAR     = 2;
   BOT_TAB_CARGAR     = 3;
   BOT_TAB_AYUDA      = 4;
   BOT_TAB_SALIR      = 5;

   BOT_DLG_SI = 1;
   BOT_DLG_NO = 2;

   BOTON_ANCHO = 96;
   BOTON_ALTO  = 24;
   BOTON_SEP_Y = 32;

   BOTMNU_INITX  = 112;
   BOTMNU_INITY  =  32;
   BOTMNU_SALIRY = 168;

   BOTTAB_INITX  = 216;
   BOTTAB_INITY  =   8;
   BOTTAB_SALIRY = 144;

   BOTDLG_INITX =  54;
   BOTDLG_SEP_X = 116;
   BOTDLG_INITY = 126;

   {-----------------------------------------------------------------------------}
   { Constantes relacionadas con las partidas guardadas.                         }
   {-----------------------------------------------------------------------------}
   MAX_PARTIDAS = 10;

   PART_INITX =  10;
   PART_INITY =  20;
   PART_INITW = 300;
   PART_INITH =  16;
   PART_SEP_Y =  18;

   PART_EST_NORMAL = 0;
   PART_EST_AVISO  = 1;
   PART_EST_METER  = 2;

   {-----------------------------------------------------------------------------}
   { Constantes relacionadas con el cursor del ratn.                            }
   {-----------------------------------------------------------------------------}
   MAX_CUR = 2;

   CUR_NORMAL = 1;
   CUR_COGER  = 2;

   CURSOR_ANCHO = 16;
   CURSOR_ALTO  = 16;

   {-----------------------------------------------------------------------------}
   { Constantes relacionadas con las teclas del teclado.                         }
   {-----------------------------------------------------------------------------}
   TECLA_ESC       = 27;
   TECLA_ENTER     = 13;
   TECLA_RETROCESO =  8;

   {-----------------------------------------------------------------------------}
   { Constantes relacionadas con los estados del juego.                          }
   {-----------------------------------------------------------------------------}
   EST_MENU       =  0;
   EST_1JUG       = 11;
   EST_2JUG       = 12;
   EST_SALVAR1    = 21;
   EST_SALVAR2    = 22;
   EST_CARGAR1    = 31;
   EST_CARGAR2    = 32;
   EST_CARGAR3    = 33;
   EST_AYUDA1     = 41;
   EST_AYUDA2     = 42;
   EST_AYUDA3     = 43;
   EST_SALIR1     = 51;
   EST_SALIR2     = 52;
   EST_SALIR3     = 53;
   EST_REINICIAR1 = 61;
   EST_REINICIAR2 = 62;
   EST_FIN        = 80;

   {-----------------------------------------------------------------------------}
   { Constantes relacionadas con los colores de la paleta.                       }
   {-----------------------------------------------------------------------------}
   COL_NEGRO       =   0; {Negro}
   COL_GRANATE     =   1; {Rojo fuerte}
   COL_VERDE       =   2; {Verde fuerte}
   COL_MARRON      =   3; {Marron}
   COL_MARINO      =   4; {Azul marino}
   COL_PURPURA     =   5; {Purpura}
   COL_AZULADO     =   6; {Azul apagado}
   COL_VERDOSO     = 245; {Azul verdoso}
   COL_GRIS_CLARO  = 247; {Gris claro}
   COL_GRIS        = 248; {Gris fuerte}
   COL_ROJO        = 249; {Rojo}
   COL_VERDE_CLARO = 250; {Verde claro}
   COL_AMARILLO    = 251; {Amarillo}
   COL_AZUL        = 252; {Azul fuerte}
   COL_ROSA        = 253; {Rosa}
   COL_TURQUESA    = 254; {Azul claro}
   COL_BLANCO      = 255; {Blanco}

   {-----------------------------------------------------------------------------}
   { Constantes relacionadas con los colores de ciertas cadenas de texto.        }
   {-----------------------------------------------------------------------------}
   COL_TXTDLG = 255; {COL_BLANCO}
   COL_TXTTIT =   2; {COL_VERDE}
   COL_TXTAYU =   6; {COL_AZULADO}
   COL_TXTEJE = 252; {COL_AZUL}
   COL_TXTCON =   3; {COL_MARRON}
   COL_TXTSAL = 255; {COL_BLANCO}
   COL_TXTCAR = 255; {COL_BLANCO}
   COL_TXTMSJ =   3; {COL_MARRON}
   COL_TXTVIC = 255; {COL_BLANCO}

   {-----------------------------------------------------------------------------}
   { Constantes relacionadas con los colores de ciertos objetos de la pantalla.  }
   {-----------------------------------------------------------------------------}
   COL_CASSEL = 249; {COL_ROJO}

   {-----------------------------------------------------------------------------}
   { Constantes relacionadas con la victoria en el juego.                        }
   {-----------------------------------------------------------------------------}
   VICTORIA_NO  = 0;
   VICTORIA_J1  = 1;
   VICTORIA_J2  = 2;
   JUEGO_TABLAS = 3;

   {-----------------------------------------------------------------------------}
   { Constantes relacionadas con los mensajes del juego al jugador.              }
   {-----------------------------------------------------------------------------}
   MSJ_NINGUNO = 0;
   MSJ_MOVINV  = 1; {Movimiento invalido}
   MSJ_JAQUEB  = 2; {Jaque al rey blanco}
   MSJ_JAQUEN  = 4; {Jaque al rey blanco}
   MSJ_MOVINVJ = 5; {Movimiento invalido por jaque mate}
   MSJ_ENROQUE = 6; {Enroque imposible, rey en jaque}
   MSJ_JAQUEMB = 7; {Jaque mate al rey blanco}
   MSJ_JAQUEMN = 8; {Jaque mate al rey blanco}

   {-----------------------------------------------------------------------------}
   { Otras constantes del juego.                                                 }
   {-----------------------------------------------------------------------------}
   TIEMPO = 200;

{********************************************************************************}
{ Tipos estructurados del juego.                                                 }
{********************************************************************************}
type
   {-----------------------------------------------------------------------------}
   { Tipos relacionados con el estado de las fichas.                             }
   {-----------------------------------------------------------------------------}
   tFicha = record
         Tipo:    byte;
         Fila:    byte;
         Columna: byte;
         Movesp:  byte;
      end;

   tFichas = array[FICHA_BLANCA..FICHA_NEGRA, 1..MAX_FICHAS_JUG] of tFicha;
   
   {-----------------------------------------------------------------------------}
   { Tipos relacionados con el arbol de bsqueda de la IA.                       }
   {-----------------------------------------------------------------------------}
   tMovFicha = record
         incc: shortint;
         incf: shortint;
      end;

   tMovsFichas = record
         PeonBlanco: array[1..MAX_MOVS_PEON] of tMovFicha;
         PeonNegro:  array[1..MAX_MOVS_PEON] of tMovFicha;
         Torre:      array[1..MAX_MOVS_TORRE] of tMovFicha;
         Caballo:    array[1..MAX_MOVS_CABALLO] of tMovFicha;         
         Alfil:      array[1..MAX_MOVS_ALFIL] of tMovFicha;
         Reina:      array[1..MAX_MOVS_REINA] of tMovFicha;
         Rey:        array[1..MAX_MOVS_REY] of tMovFicha;
         NumMovs:    array[FICHA_MUERTA..FICHA_REY] of byte;
      end;

   {-----------------------------------------------------------------------------}
   { Tipo que almacena todos los datos relacionados con las texturas.            }
   {-----------------------------------------------------------------------------}
   tTexturas = record
         CClave: byte;

         Botones: tImagen;
         Fichas:  tImagen;
         Menu:    tImagen;
         Tablero: tImagen;
         Cursor:  tImagen;

         PosBotones: array[1..MAX_BOTONES] of tRect;
         MnuBotones: array[1..MNU_BOTONES] of tRect;
         TabBotones: array[1..TAB_BOTONES] of tRect;
         DlgBotones: array[1..DLG_BOTONES] of tRect;
         PosFichas:  array[1..MAX_FICHAS, FICHA_BLANCA..FICHA_NEGRA] of tRect;
         PosCursor:  array[1..MAX_CUR] of tRect;
      end;

   {-----------------------------------------------------------------------------}
   { Tipo que almacena todo los datos de la partida actual.                      }
   {-----------------------------------------------------------------------------}
   tJuego = record
         JaqueB:   boolean;
         JaqueN:   boolean;
         Victoria: byte;
         Mensaje:  byte;
         Estado:   byte;
         Turno:    byte;
         Fichas:   tFichas;
         selx:     byte;
         sely:     byte;
         NoPeon:   boolean;
         EstadoIA: byte;
      end;

   {-----------------------------------------------------------------------------}
   { Tipo que almacena todo los datos relacionados con las partidas guardadas.   }
   {-----------------------------------------------------------------------------}
   tPartidas = record
         Estado:  byte;
         Auxcad:  string;
         Parsel:  byte;
         Nombre:  array[1..MAX_PARTIDAS] of string;
         Fichero: array[1..MAX_PARTIDAS] of string;
      end;

   tRegPartida = record
         Nombre: string;
         Juego:  tJuego;
      end;

   tFichPartida = file of tRegPartida;

   {-----------------------------------------------------------------------------}
   { Tipo que almacena todos los datos del programa.                             }
   {-----------------------------------------------------------------------------}
   tNucleo = record
         Juego:      tJuego;
         MovsFichas: tMovsFichas;
         Partidas:   tPartidas;
         Texturas:   tTexturas;
         Fuente:     tFuente;
         Paleta:     tPaleta;
         Raton:      tRaton;

         Errlog:     text;
      end;


{********************************************************************************}
{ Funcines que se ejecutan al principio o al final del juego.                   }
{********************************************************************************}
procedure CargarTexturas (var Nucleo: tNucleo);
procedure InitPartidas   (var Nucleo: tNucleo);
function  InitJuego      (var Nucleo: tNucleo): boolean;
procedure FinJuego       (var Nucleo: tNucleo);
procedure DoDelay        (ms: word);


{********************************************************************************}
implementation {parte privada}
{********************************************************************************}


{********************************************************************************}
{ Funcin que emula un delay.                                                    }
{********************************************************************************}
procedure DoDelay (ms: word);
var
   ti, tact, tf: dword;
begin
   {-----------------------------------------------------------------------------}
   { Tomamos los valores de tiempo.                                              }
   {-----------------------------------------------------------------------------}
   ti   := DarTiempo;
   tf   := DarTiempo + ms;
   tact := ti;

   {-----------------------------------------------------------------------------}
   { Mientras el tiempo actual, sea menor que el tiempo final, estaremos         }
   { esperando. Y el tiempo inicial tiene que ser menor o igual que el tiempo    }
   { actual, para que en caso de que el reloj de la vuelta no se cuelgue el      }
   { programa, esperando que pasel varios millones de milisegundos.              }
   {-----------------------------------------------------------------------------}
   while (tact < tf) and (ti <= tact) do
      tact := DarTiempo;
end;

{********************************************************************************}
{ Funcin que carga las texturas del juego.                                      }
{********************************************************************************}
procedure CargarTexturas (var Nucleo: tNucleo);
var
   i, j: integer;
begin
   {-----------------------------------------------------------------------------}
   { Carga de las imagenes.                                                      }
   {-----------------------------------------------------------------------------}
   M13hImagenCargarFichero('img\Menu.bmp', Nucleo.Texturas.Menu, Nucleo.Paleta);
   M13hImagenCargar('img\Tablero.bmp', Nucleo.Texturas.Tablero);
   M13hImagenCargar('img\Botones.bmp', Nucleo.Texturas.Botones);
   M13hImagenCargar('img\Fichas.bmp', Nucleo.Texturas.Fichas);
   M13hImagenCargar('img\Cursor.bmp', Nucleo.Texturas.Cursor);

   {-----------------------------------------------------------------------------}
   { Carga de la paleta.                                                         }
   {-----------------------------------------------------------------------------}
   M13hPaleta24a18(Nucleo.Paleta);
   M13hPaletaCargar(Nucleo.Paleta);

   {-----------------------------------------------------------------------------}
   { Seleccionamos el color clave.                                               }
   {-----------------------------------------------------------------------------}
   Nucleo.Texturas.CClave := M13hImagenDarPixel(0, 0, Nucleo.Texturas.Fichas);

   {-----------------------------------------------------------------------------}
   { Establecer las regiones de los botones.                                     }
   {-----------------------------------------------------------------------------}
   for i := 1 to MAX_BOTONES do
    begin
      Nucleo.Texturas.PosBotones[i].x := 0;
      Nucleo.Texturas.PosBotones[i].y := BOTON_ALTO * (i - 1);
      Nucleo.Texturas.PosBotones[i].w := BOTON_ANCHO;
      Nucleo.Texturas.PosBotones[i].h := BOTON_ALTO;
    end;

   {-----------------------------------------------------------------------------}
   { Establecer las regiones de los botones del men.                            }
   {-----------------------------------------------------------------------------}
   for i := 1 to MNU_BOTONES do
    begin
      Nucleo.Texturas.MnuBotones[i].x := BOTMNU_INITX;
      Nucleo.Texturas.MnuBotones[i].y := BOTON_SEP_Y * (i - 1) + BOTMNU_INITY;
      Nucleo.Texturas.MnuBotones[i].w := BOTON_ANCHO;
      Nucleo.Texturas.MnuBotones[i].h := BOTON_ALTO;
    end;

   Nucleo.Texturas.MnuBotones[BOT_MNU_SALIR].y := BOTMNU_SALIRY;

   {-----------------------------------------------------------------------------}
   { Establecer las regiones de los botones del tablero.                         }
   {-----------------------------------------------------------------------------}
   for i := 1 to TAB_BOTONES do
    begin
      Nucleo.Texturas.TabBotones[i].x := BOTTAB_INITX;
      Nucleo.Texturas.TabBotones[i].y := BOTON_SEP_Y * (i - 1) + BOTTAB_INITY;
      Nucleo.Texturas.TabBotones[i].w := BOTON_ANCHO;
      Nucleo.Texturas.TabBotones[i].h := BOTON_ALTO;
    end;

   Nucleo.Texturas.TabBotones[BOT_TAB_SALIR].y := BOTTAB_SALIRY;

   {-----------------------------------------------------------------------------}
   { Establecer las regiones de los botones del cuadro de dilogo.               }
   {-----------------------------------------------------------------------------}
   for i := 1 to DLG_BOTONES do { SI = (54, 126), NO = (170, 126) }
    begin
      Nucleo.Texturas.DlgBotones[i].x := BOTDLG_SEP_X * (i - 1) + BOTDLG_INITX;
      Nucleo.Texturas.DlgBotones[i].y := BOTDLG_INITY;
      Nucleo.Texturas.DlgBotones[i].w := BOTON_ANCHO;
      Nucleo.Texturas.DlgBotones[i].h := BOTON_ALTO;
    end;

   {-----------------------------------------------------------------------------}
   { Establecer las regiones de las fichas.                                      }
   {-----------------------------------------------------------------------------}
   for i := 1 to MAX_FICHAS do
      for j := FICHA_BLANCA to FICHA_NEGRA do
       begin
         Nucleo.Texturas.PosFichas[i, j].x := FICHA_ANCHO * (i - 1);
         Nucleo.Texturas.PosFichas[i, j].y := FICHA_ALTO * (j - 1);
         Nucleo.Texturas.PosFichas[i, j].w := FICHA_ANCHO;
         Nucleo.Texturas.PosFichas[i, j].h := FICHA_ALTO;
       end;

   {-----------------------------------------------------------------------------}
   { Establecer las regiones de los cursores.                                    }
   {-----------------------------------------------------------------------------}
   for i := 1 to MAX_CUR do
    begin
      Nucleo.Texturas.PosCursor[i].x := CURSOR_ANCHO * (i - 1);
      Nucleo.Texturas.PosCursor[i].y := 0;
      Nucleo.Texturas.PosCursor[i].w := CURSOR_ANCHO;
      Nucleo.Texturas.PosCursor[i].h := CURSOR_ALTO;
    end;

   {-----------------------------------------------------------------------------}
   { Carga de la fuente.                                                         }
   {-----------------------------------------------------------------------------}
   FuenteCargar('img\Letras.bmp', Nucleo.Fuente);
end;

{********************************************************************************}
{ Funcin que inicializa los datos de la partidas guardadas.                     }
{********************************************************************************}
procedure InitPartidas (var Nucleo: tNucleo);
var
   i: integer;
   aux: string;
   f: tFichPartida;
   reg: tRegPartida;
begin
   {-----------------------------------------------------------------------------}
   { Inicializamos las variables que tienen que ver con el men.                 }
   {-----------------------------------------------------------------------------}
   Nucleo.Partidas.Estado := PART_EST_NORMAL;
   Nucleo.Partidas.Auxcad := '';

   {-----------------------------------------------------------------------------}
   { Inicializamos los nombres de los ficheros.                                  }
   {-----------------------------------------------------------------------------}
   for i := 1 to MAX_PARTIDAS do
    begin
      str(i - 1, aux);
      Nucleo.Partidas.Fichero[i] := 'save\Save0' + aux + '.sav';
    end;

   {-----------------------------------------------------------------------------}
   { Inicializamos los nombres de las partidas.                                  }
   {-----------------------------------------------------------------------------}
   for i := 1 to MAX_PARTIDAS do
    begin
      {--------------------------------------------------------------------------}
      { Asignamos el fichero a la variable f.                                    }
      {--------------------------------------------------------------------------}
      assign(f, Nucleo.Partidas.Fichero[i]);

      {--------------------------------------------------------------------------}
      { Lo abrimos para leer.                                                    }
      {--------------------------------------------------------------------------}
      {$I-}
      reset(f);
      {$I+}

      {--------------------------------------------------------------------------}
      { Si no hay ningn fallo leemos su contenido y pillamos el nombre.         }
      {--------------------------------------------------------------------------}
      if IOResult = 0 then
       begin
         if filesize(f) > 0 then
          begin
            read(f, reg);
            Nucleo.Partidas.Nombre[i] := reg.Nombre;
          end
         else
            Nucleo.Partidas.Nombre[i] := '---';

         close(f);
       end
      {--------------------------------------------------------------------------}
      { En caso de error inicializamos el nombre con una cadena de "error".      }
      {--------------------------------------------------------------------------}
      else
       begin
         Nucleo.Partidas.Nombre[i] := '---';
         rewrite(f);
         close(f);
       end;
    end;
end;

{********************************************************************************}
{ Funcin que inicializa los datos de la partidas guardadas.                     }
{********************************************************************************}
procedure InitMovsFichas (var Nucleo: tNucleo);
var
   i: byte;
begin
   {-----------------------------------------------------------------------------}
   { Movimientos del pen blanco.                                                }
   {-----------------------------------------------------------------------------}
   Nucleo.MovsFichas.PeonBlanco[1].incf := +1;
   Nucleo.MovsFichas.PeonBlanco[1].incc := -1; { Comer a la izquierda }
   Nucleo.MovsFichas.PeonBlanco[2].incf := +1;
   Nucleo.MovsFichas.PeonBlanco[2].incc :=  0; { Mover hacia delante }
   Nucleo.MovsFichas.PeonBlanco[3].incf := +1;
   Nucleo.MovsFichas.PeonBlanco[3].incc := +1; { Comer a la derecha }
   Nucleo.MovsFichas.PeonBlanco[4].incf := +2;
   Nucleo.MovsFichas.PeonBlanco[4].incc :=  0; { Mover hacia delante }
   
   {-----------------------------------------------------------------------------}
   { Movimientos del pen negro.                                                 }
   {-----------------------------------------------------------------------------}
   Nucleo.MovsFichas.PeonNegro[1].incf := -1;
   Nucleo.MovsFichas.PeonNegro[1].incc := -1; { Comer a la izquierda }
   Nucleo.MovsFichas.PeonNegro[2].incf := -1;
   Nucleo.MovsFichas.PeonNegro[2].incc :=  0; { Mover hacia delante }
   Nucleo.MovsFichas.PeonNegro[3].incf := -1;
   Nucleo.MovsFichas.PeonNegro[3].incc := +1; { Comer a la derecha }
   Nucleo.MovsFichas.PeonNegro[4].incf := -2;
   Nucleo.MovsFichas.PeonNegro[4].incc :=  0; { Mover hacia delante }
   
   {-----------------------------------------------------------------------------}
   { Movimientos del caballo.                                                    }
   {-----------------------------------------------------------------------------}
   Nucleo.MovsFichas.Caballo[1].incf := +2;
   Nucleo.MovsFichas.Caballo[1].incc := -1;
   
   Nucleo.MovsFichas.Caballo[2].incf := +2;
   Nucleo.MovsFichas.Caballo[2].incc := +1;
   
   Nucleo.MovsFichas.Caballo[3].incf := +1;
   Nucleo.MovsFichas.Caballo[3].incc := -2;
   
   Nucleo.MovsFichas.Caballo[4].incf := +1;
   Nucleo.MovsFichas.Caballo[4].incc := +2;
   
   Nucleo.MovsFichas.Caballo[5].incf := -1;
   Nucleo.MovsFichas.Caballo[5].incc := -2;
   
   Nucleo.MovsFichas.Caballo[6].incf := -1;
   Nucleo.MovsFichas.Caballo[6].incc := +2;
   
   Nucleo.MovsFichas.Caballo[7].incf := -2;
   Nucleo.MovsFichas.Caballo[7].incc := -1;
   
   Nucleo.MovsFichas.Caballo[8].incf := -2;
   Nucleo.MovsFichas.Caballo[8].incc := +1;
   
   {-----------------------------------------------------------------------------}
   { Movimientos del rey.                                                        }
   {-----------------------------------------------------------------------------}
   Nucleo.MovsFichas.Rey[1].incf := +1;
   Nucleo.MovsFichas.Rey[1].incc := -1;
   
   Nucleo.MovsFichas.Rey[2].incf := +1;
   Nucleo.MovsFichas.Rey[2].incc :=  0;
   
   Nucleo.MovsFichas.Rey[3].incf := +1;
   Nucleo.MovsFichas.Rey[3].incc := +1;
   
   Nucleo.MovsFichas.Rey[4].incf :=  0;
   Nucleo.MovsFichas.Rey[4].incc := -1;
   
   Nucleo.MovsFichas.Rey[5].incf :=  0;
   Nucleo.MovsFichas.Rey[5].incc := +1;
   
   Nucleo.MovsFichas.Rey[6].incf := -1;
   Nucleo.MovsFichas.Rey[6].incc := -1;
   
   Nucleo.MovsFichas.Rey[7].incf := -1;
   Nucleo.MovsFichas.Rey[7].incc :=  0;
   
   Nucleo.MovsFichas.Rey[8].incf := -1;
   Nucleo.MovsFichas.Rey[8].incc := +1;
   
   Nucleo.MovsFichas.Rey[9].incf :=  0;
   Nucleo.MovsFichas.Rey[9].incc := +2;
   
   Nucleo.MovsFichas.Rey[10].incf :=  0;
   Nucleo.MovsFichas.Rey[10].incc := -2;


   for i := 1 to 7 do
    begin
      {--------------------------------------------------------------------------}
      { Movimientos de la torre.                                                 }
      {--------------------------------------------------------------------------}
      Nucleo.MovsFichas.Torre[i].incf := +i;
      Nucleo.MovsFichas.Torre[i].incc :=  0;
      
      Nucleo.MovsFichas.Torre[i+7].incf := -i;
      Nucleo.MovsFichas.Torre[i+7].incc :=  0;
      
      Nucleo.MovsFichas.Torre[i+14].incf :=  0;
      Nucleo.MovsFichas.Torre[i+14].incc := +i;
      
      Nucleo.MovsFichas.Torre[i+21].incf :=  0;
      Nucleo.MovsFichas.Torre[i+21].incc := -i;

      {--------------------------------------------------------------------------}
      { Movimientos del alfil.                                                   }
      {--------------------------------------------------------------------------}
      Nucleo.MovsFichas.Alfil[i].incf := +i;
      Nucleo.MovsFichas.Alfil[i].incc := +i;
      
      Nucleo.MovsFichas.Alfil[i+7].incf := -i;
      Nucleo.MovsFichas.Alfil[i+7].incc := +i;
      
      Nucleo.MovsFichas.Alfil[i+14].incf := +i;
      Nucleo.MovsFichas.Alfil[i+14].incc := -i;
      
      Nucleo.MovsFichas.Alfil[i+21].incf := -i;
      Nucleo.MovsFichas.Alfil[i+21].incc := -i;
      
      {--------------------------------------------------------------------------}
      { Movimientos de la reina.                                                 }
      {--------------------------------------------------------------------------}
      Nucleo.MovsFichas.Reina[i].incf := +i;
      Nucleo.MovsFichas.Reina[i].incc :=  0;
      
      Nucleo.MovsFichas.Reina[i+7].incf := -i;
      Nucleo.MovsFichas.Reina[i+7].incc :=  0;
      
      Nucleo.MovsFichas.Reina[i+14].incf :=  0;
      Nucleo.MovsFichas.Reina[i+14].incc := +i;
      
      Nucleo.MovsFichas.Reina[i+21].incf :=  0;
      Nucleo.MovsFichas.Reina[i+21].incc := -i;
      
      Nucleo.MovsFichas.Reina[i+28].incf := +i;
      Nucleo.MovsFichas.Reina[i+28].incc := +i;
      
      Nucleo.MovsFichas.Reina[i+35].incf := -i;
      Nucleo.MovsFichas.Reina[i+35].incc := +i;
      
      Nucleo.MovsFichas.Reina[i+42].incf := +i;
      Nucleo.MovsFichas.Reina[i+42].incc := -i;
      
      Nucleo.MovsFichas.Reina[i+49].incf := -i;
      Nucleo.MovsFichas.Reina[i+49].incc := -i;
    end;

   {-----------------------------------------------------------------------------}
   { Almacenamos la cantidad de movimientos mximos de cada tipo de pieza.       }
   {-----------------------------------------------------------------------------}
   Nucleo.MovsFichas.NumMovs[FICHA_MUERTA]  := 0;
   Nucleo.MovsFichas.NumMovs[FICHA_PEON]    := MAX_MOVS_PEON;
   Nucleo.MovsFichas.NumMovs[FICHA_TORRE]   := MAX_MOVS_TORRE;
   Nucleo.MovsFichas.NumMovs[FICHA_CABALLO] := MAX_MOVS_CABALLO;
   Nucleo.MovsFichas.NumMovs[FICHA_ALFIL]   := MAX_MOVS_ALFIL;
   Nucleo.MovsFichas.NumMovs[FICHA_REINA]   := MAX_MOVS_REINA;
   Nucleo.MovsFichas.NumMovs[FICHA_REY]     := MAX_MOVS_REY;
end;

{********************************************************************************}
{ Funcin que inicializa los datos del juego.                                    }
{********************************************************************************}
function InitJuego (var Nucleo: tNucleo): boolean;
var
   Resultado: boolean;
begin
   Resultado := false;

   assign(Nucleo.Errlog, 'Log.txt');
   rewrite(Nucleo.Errlog);

   {-----------------------------------------------------------------------------}
   { Activar el modo13h.                                                         }
   {-----------------------------------------------------------------------------}
   if M13hActivar then
    begin
      {--------------------------------------------------------------------------}
      { Activar el ratn, para luego ocultar el cursor, limitar su area de       }
      { movimiento y actualizar el estado del ratn. Luego cargamos las texturas }
      { e indicamos el estado actual del juego, para que vaya al men.           }
      {--------------------------------------------------------------------------}
      if RatonActivar > 0 then
       begin
         RatonOcultar;
         RatonLimitar(0, 0, 304, 184);
         RatonEstado(Nucleo.Raton);

         CargarTexturas(Nucleo);

         InitPartidas(Nucleo);

         InitMovsFichas(Nucleo);

         Nucleo.Juego.Estado := EST_MENU;

         Resultado := true;
       end
      {--------------------------------------------------------------------------}
      { Si no se puede activar el ratn lanzar un aviso de error.                }
      {--------------------------------------------------------------------------}
      else
       begin
         M13hDesactivar;
         writeln('No se ha podido activar el ratn!');
         readkey;
       end;
    end
   {-----------------------------------------------------------------------------}
   { Si no se pudo activar el modo13h lanzar aviso de error.                     }
   {-----------------------------------------------------------------------------}
   else
    begin
      writeln('No se ha podido cambiar el modo de pantalla!');
      readkey;
    end;

   {-----------------------------------------------------------------------------}
   { Devolver el resultado final de la funcin.                                  }
   {-----------------------------------------------------------------------------}
   InitJuego := Resultado;
end;

{********************************************************************************}
{ Funcin que borra los datos del juego.                                         }
{********************************************************************************}
procedure FinJuego (var Nucleo: tNucleo);
begin
   {-----------------------------------------------------------------------------}
   { Borra las imagenes cargadas.                                                }
   {-----------------------------------------------------------------------------}
   M13hBorrarImagen(Nucleo.Texturas.Menu);
   M13hBorrarImagen(Nucleo.Texturas.Tablero);
   M13hBorrarImagen(Nucleo.Texturas.Botones);
   M13hBorrarImagen(Nucleo.Texturas.Fichas);
   M13hBorrarImagen(Nucleo.Texturas.Cursor);

   {-----------------------------------------------------------------------------}
   { Borra la fuente cargada.                                                    }
   {-----------------------------------------------------------------------------}
   FuenteDescargar(Nucleo.Fuente);

   {-----------------------------------------------------------------------------}
   { Desactiva el modo13h.                                                       }
   {-----------------------------------------------------------------------------}
   M13hDesactivar;

   close(Nucleo.Errlog);
end;

{********************************************************************************}
end.
{********************************************************************************}